#!/bin/bash

################################# IMPORTANT NOTE ###########################################################
# 				
#  USAGE:
#
#	Changing the script requires building the following modules/packages
#	since the script is called from $EXEPATH during migrate export
#
#		
#  						
#		1. ngm_upgrade
#		2. ngm_upgrade_wrapper											
#		3. upgrade_tools						
#								
#
# 
#
############################################################################################################

# kill switch
if [ "$SKIP_LAAS_MIGRATE" == "1" ]; then
	exit 0
fi 
# COMMON CONSTS
LAAS_TARGETCONFIGURATION_PATH=$EXPORTERDIR/targets/laas/targetConfiguration.xml
RFL_CONFIG_PATH=$RTDIR/conf/rflConfig.xml

# EXPORT CONSTS
LAAS_INGEST_REG_KEY="LaaSIngestEnabled"
LAAS_QUERY_REG_KEY="LaaSEnabled"
LAAS_OPTIMIZATION_REG_KEY="LaaSOptimizationEnabled"
LAAS_REG_FILE="laas-reg"

# IMPORT CONSTS
# laas.properties is copied in migrate.conf
ORIG_LAAS_PROPERTIES=$RTDIR/laas/conf/laas.properties
LAAS_ACTIVATION=$RTDIR/scripts/laas-activation.sh
LAAS_EXPORTER_DIR=$EXPORTERDIR/targets/laas
INGESTION_MODE="ingestion"
QUERY_MODE="query"
ALL_MODE="all"
OPTIMIZATION_MODE="optimization"
PUBLIC_CLOUD_CERT=$CPDIR/conf/ca-bundle-public-cloud.crt
CA_BUNDLE_CERT=$CPDIR/conf/ca-bundle.crt
KEY_NOT_EXIST=255

function exitOnFailureMessage(){
	statusCode=$1
	echo "Failed to run laas-migrate script, exit with status code: [$statusCode] "
}


function usage() {
	echo "Usage:  $0 export <export-path> | import  <import-path>"	      
}

function title() {
	echo "------------------------------------------------"
	echo "$1"
	echo "------------------------------------------------"
	return 0
}

function isLaaSConfigured(){

	if [ "$LAAS_INGEST_KEY" == "$KEY_NOT_EXIST" ] && [ "$LAAS_QUERY_KEY" == "$KEY_NOT_EXIST" ] && [ "$LAAS_OPTIMIZATION_KEY" == "$KEY_NOT_EXIST" ]; then
		return 1
	fi
	return 0
}


############## EXPORT FUNCTIONS ##############

function readRegisry() {
	registryName=$1
	laasRegistry=`$CPDIR/bin/cpprod_util CPPROD_GetValue "Reporting Module" $registryName 1`
	status=$?
	if [ $status -ne 0 ]; then
		echo "Could not read registry value, setting $registryName to $KEY_NOT_EXIST"
		registryEnabled=$KEY_NOT_EXIST
		return $registryEnabled	 
	fi
	if [[ "$laasRegistry" == "1" ]]; then
		registryEnabled=1
	elif [[ "$laasRegistry" == "0" ]]; then
		registryEnabled=0
	else
		# -1 if the registry key does not exist
		registryEnabled=$KEY_NOT_EXIST
	fi
	echo "registry $registryName was set to $registryEnabled"
	return $registryEnabled
}

function readRegisryFile(){

	readRegisry "LaaSIngestEnabled" 
	LAAS_INGEST_KEY=$?
	
	readRegisry "LaaSEnabled"
	LAAS_QUERY_KEY=$?
	
	readRegisry "LaaSOptimizationEnabled"
	LAAS_OPTIMIZATION_KEY=$?
}

function writeRegistryFile(){
	
	echo "LAAS_INGEST_KEY=$LAAS_INGEST_KEY" > $LAAS_REGISTRY_CONFIG_PATH
	echo "LAAS_QUERY_KEY=$LAAS_QUERY_KEY" >> $LAAS_REGISTRY_CONFIG_PATH
	echo "LAAS_OPTIMIZATION_KEY=$LAAS_OPTIMIZATION_KEY" >> $LAAS_REGISTRY_CONFIG_PATH
}

function getValue(){
	tagName=$1
	fileName=$2
	context=$3
	value=`grep -oP "(?<=<$tagName>).*?(?=</$tagName>)" $fileName`
	#value=`grep -w $tagName $fileName | sed 's/\s*<'$tagName'>\(.*\)<.*>/\1/'`
	status=$?
	if [ $status -ne 0 ]; then 
		echo "Error while extracting $tagName for $context from file $fileName, message: $value"
		echo "Setting value to empty string"
		$value=""
	fi
	echo $value
}


function laasExport(){

##### Export Main #####
	
	if [ ! -d "$LAAS_REPOSITORY" ]; then
		mkdir -p $LAAS_REPOSITORY
		status=$?
		if [ $status -ne 0 ]; then
			exitOnFailureMessage $status
			exit $status
		fi
	fi
	
	# Reading registry keys 
	title "Going to read laas registry"
	readRegisryFile
	
	title "Checking if LaaS is configured on machine "
	if ! isLaaSConfigured; then
		echo " LaaS is not configured on source machine, exiting script..."
		exit 0
	fi
	
	# Writing registry keys 
	title "Going to write laas registry keys to file"
	writeRegistryFile
	
	title "LaaS export operation has been completed successfully"
	exit 0
}

############## IMPORT FUNCTION ##############

function laasActivation(){
	mode=$1
	key=$2
	if [ "$key" == "0" ]; then
		command="disable"
	elif [ "$key" == "1" ]; then
		command="enable"
	else
		echo "ERROR: laas-activation script won't run, registry key $key is not valid"
		cleanup
		exitOnFailureMessage 1
		exit 1
	fi
	
	title "Running laas-activation script..."
	$LAAS_ACTIVATION $command $mode
	status=$?
	echo "$command registry to $mode mode with status code: [$status]"
	if [ $status -ne 0 ]; then
		echo "Failed to run laas-activation script"
		cleanup
		exitOnFailureMessage $status
		exit $status
	fi
}


function laasImport(){
IMPORT_LAAS_PROPERTIES=$LAAS_REPOSITORY/laas.properties

##### Import Main #####
			
	if [ ! -f $LAAS_REGISTRY_CONFIG_PATH ]; then
		echo "LaaS should not be configured on the machine, $LAAS_REG_FILE file does not exist, exiting script..."
		exit 0
	fi
	
	# Reading registry
	. $LAAS_REGISTRY_CONFIG_PATH
	if ! isLaaSConfigured; then
	# add message that we done and context
		echo "LaaS should not be configured on the machine, laas registry keys were not found, exiting script..."
		exit 0
	fi
	
		
	###############################################################################################
	############################ Activating LaaS according to mode ################################
	
	if [ "$LAAS_OPTIMIZATION_KEY" == "$KEY_NOT_EXIST" ] && [ "$LAAS_QUERY_KEY" == "1" ] && [ "$LAAS_INGEST_KEY" == "$KEY_NOT_EXIST" ] ; then
	# This case is for MaaS where the only flag is 'LaaSEnabled'	
		LAAS_OPTIMIZATION_KEY=1
		LAAS_INGEST_KEY=1
	fi
	
	if [[ "$LAAS_INGEST_KEY" != $KEY_NOT_EXIST ]]; then
		laasActivation	$INGESTION_MODE $LAAS_INGEST_KEY
	fi
	if [[ "$LAAS_QUERY_KEY" != $KEY_NOT_EXIST ]]; then
		laasActivation $QUERY_MODE $LAAS_QUERY_KEY
	fi
	if [[ "$LAAS_OPTIMIZATION_KEY" != $KEY_NOT_EXIST ]]; then
		laasActivation $OPTIMIZATION_MODE $LAAS_OPTIMIZATION_KEY
	fi
	
	title "LaaS post import operation has been completed successfully"
	exit 0
}

### MAIN ###
if [ $# -ne 2 ]; then
	usage
	exit 1
fi

UPGRADE_MODE=$1
LAAS_REPOSITORY=$2
LAAS_REGISTRY_CONFIG_PATH=$LAAS_REPOSITORY/$LAAS_REG_FILE
LAAS_CONFIGURATION_PATH=$LAAS_REPOSITORY/laas.properties


if [ -z "$LAAS_REPOSITORY" ]; then
	echo "LaaS upgrade directory was not provided"
	exit 1
fi

if [ "$UPGRADE_MODE" == "export" ]; then
	laasExport
elif [ "$UPGRADE_MODE" == "import" ]; then
	laasImport
else
	echo "Invalid argument: $1"
	usage
	exit 1
fi





