#!/bin/sh

NC='\033[0m' # No Color
DATA_FILE=$MDS_FWDIR/log/status_table_in_sys_domain.txt
OBJECT_SIC_NAME=`grep "MySICname" /$CPDIR/registry/HKLM_registry.data | sed -r s/'.*[CN,cn]=(.+),.*'/\\\1/`
OBJECT_PRIMARY_SIC_NAME=cp_mgmt
CURRENT_MACHINE=`uname -n`
uniq_ext=`/bin/date '+%H%M%S' | /bin/sed 's/ //g'`
MYFOLDERNAME=/tmp/stat$uniq_ext

echo ""
echo ""
usage()
{
  echo ""	
  echo "usage: calculate_sync_state.sh [optional flags]"
  echo ""
  echo "Description: "
  echo "Calculate number of transaction sync to current peer, for System domain only (MDS)"	 
  echo "Optional flags:"
  echo "calculate_sync_state.sh [filename]"
  echo "calculate_sync_state.sh [filename] [MDS name]"
  echo ""
}


if [ "X$MDSDIR" = "X" ]; then
    echo "Error: This utility can only be executed in Multi-Domain Security Management."
    exit 2
fi

if [ ! -z "$1" ]
then
	if [ "X$1" = "X-h" ]
	then
		usage
		exit 2
	fi
        DATA_FILE=$1
	echo "Status information file: $DATA_FILE"
else
	echo "Extracting HA status information for System domain..."
	rm $DATA_FILE >& /dev/null
        $MDS_FWDIR/scripts/reload_env_vars.sh -e ""WRITE_STATUSES_TABLE=1"" >& /dev/null
fi

if [ ! -z "$2" ]
then
        CURRENT_MACHINE=$2
        echo "Scaning status for peer: $CURRENT_MACHINE"
else
        if [ ! -n "$OBJECT_SIC_NAME" ]
        then
                echo "ERROR: cannot resolve host name"
                usage
                exit 1
        fi
        if [ "$OBJECT_SIC_NAME" = "$OBJECT_PRIMARY_SIC_NAME" ]
        then
                #OBJECT_SIC_NAME=`grep "MySICname" /$CPDIR/registry/HKLM_registry.data | sed -r s/'.*[O,o]=(.+)[.][.].*'/\\\1/`
                OBJECT_SIC_NAME=$(psql_client cpm postgres -c "select name from cpnetworkobject_data where sicname like '%cp_mgmt,%' and not deleted and dlesession=0" | head -n 3 | tail -1| tr -d ' ')

                if [ ! -n "$OBJECT_SIC_NAME"  ]
                then
                        echo "ERROR: cannot resolve host name"
                        usage
                        exit 1
                fi
                if [[ "$OBJECT_SIC_NAME" =~ "cp_mgmt" ]]
                then
                        echo "Error: cannot resolve host name"
                        usage
                        exit 1
                fi
	else
        	OBJECT_SIC_NAME=`grep "MySICname" /$CPDIR/registry/HKLM_registry.data | sed -r s/'.*[CN,cn]=cp_mgmt_(.+),.*'/\\\1/`
	fi
	CURRENT_MACHINE=$OBJECT_SIC_NAME
fi

echo "Parsing status information..."
sleep 1

if [ ! -f "$DATA_FILE" ]
then
	echo "File not found - $DATA_FILE"
        tput setf 6
        echo "Some peers have not completed sync to current MDS/MLM"
        echo -e "${NC}"
	usage
	exit 1
fi

RECET_UPDATE_TIME=`ls -l $DATA_FILE | awk '{printf "%s %s %s \n", $6, $7, $8;exit;}'`
echo "Status recently updated on : $RECET_UPDATE_TIME"
sleep 2

mkdir -p $MYFOLDERNAME
#parse all peer names, then transpose
cat $DATA_FILE | grep -v "+------------------------------------+" | grep $CURRENT_MACHINE | head -1 > $MYFOLDERNAME/row1
cat $MYFOLDERNAME/row1 | tr '|'  ' ' |awk '
{ 
    for (i=1; i<=NF; i++)  {
        a[NR,i] = $i
    }
}
NF>p { p = NF }
END {    
    for(j=1; j<=p; j++) {
        str=a[1,j]
        for(i=2; i<=NR; i++){
            str=str" "a[i,j];
        }
        print str
    }
}'  > $MYFOLDERNAME/col1

NUM_OF_PEERS=$(cat $MYFOLDERNAME/col1 | wc -l)

if [ $NUM_OF_PEERS -lt 1 ]
then
	echo "No data found for for peer: $CURRENT_MACHINE"
	exit 1
fi

cat $DATA_FILE | grep -v "+------------------------------------+" | grep $CURRENT_MACHINE | tail -1 > $MYFOLDERNAME/row2
#tarns=$(cat $MYFOLDERNAME/row2 | tr '|'  ' ' | tr -d '/')
cat $MYFOLDERNAME/row2 | tr '|'  ' ' | tr -d '/' |awk '
{ 
    for (i=1; i<=NF; i++)  {
        a[NR,i] = $i
    }
}
NF>p { p = NF }
END {    
    for(j=1; j<=p; j++) {
        str=a[1,j]
        for(i=2; i<=NR; i++){
            str=str" "a[i,j];
        }
        print str
    }
}'> $MYFOLDERNAME/col2


PRINT_PEER_NAME=true
COUNTER=0
FOUND_LAGGING_PEER=false
pattern='^\{?[A-Z0-9a-z]{8}-[A-Z0-9a-z]{4}-[A-Z0-9a-z]{4}-[A-Z0-9a-z]{4}-[A-Z0-9a-z]{12}\}?$'
rm -f -- $MDS_FWDIR/log/not_sync_machine_peers.txt
echo ======================================================================
for  ((i = 0; i <= $NUM_OF_PEERS; i++))
do
    read line1 < $MYFOLDERNAME/col2
    sed -i '1d' $MYFOLDERNAME/col2
    if $PRINT_PEER_NAME
    then
        echo Transaction synced to Peer: $line1
        PRINT_PEER_NAME=false
        continue
    fi
    read line2 < $MYFOLDERNAME/col2
    sed -i '1d' $MYFOLDERNAME/col2
    read line3 < $MYFOLDERNAME/col1
    sed -i '1d' $MYFOLDERNAME/col1
    if [[ "$line3" =~ $pattern ]]; then
	continue 
    fi

    if [ $line2 -eq -1 -o $line2 -lt $line1 ]
	then
	   printf '%s \t\t          N/A \n' $line3 >> $MYFOLDERNAME/result
	   FOUND_LAGGING_PEER=true
	   printf '%s\n' $line3 >> $MDS_FWDIR/log/not_sync_machine_peers.txt
	else
          printf '%s \t\t %s / %s \n' $line3 $line1 $line2 >> $MYFOLDERNAME/result
    fi


    if [ $line1 -lt $line2 ]
	then
	  FOUND_LAGGING_PEER=true
	  printf '%s\n' $line3 >> $MDS_FWDIR/log/not_sync_machine_peers.txt
    fi
done

cat $MYFOLDERNAME/result | column -t
echo ""

RES=$FOUND_LAGGING_PEER
STATUS=0

if  $RES ; then
        tput setf 6
	echo "Some peers have not completed sync to current MDS/MLM"
        echo -e "${NC}"
		STATUS=11
else
        tput setf 2
        echo "Sync data complete for current MDS/MLM"
        echo -e "${NC}"
fi
echo ======================================================================
rm -rf $MYFOLDERNAME/
exit $STATUS