#!/bin/sh

JQ=${CPDIR}/jq/jq
SED=/bin/sed
ECHO=/bin/echo

mode=$1
build=$2

internal_error_msg=$'Internal error. Please download the latest Deployment Agent or contact Check Point Support.\n'
timeout_error_msg="Failed to retrieve Upgrade Tools package from Check Point Download Center (timed out). To download the package manually, refer to sk135172. In order to run again without reaching the Check Point Download Center, run $mode process using -skip_upgrade_tools_check flag"
outdated_error_msg="Your Upgrade Tools package is not up-to-date or failed to be retrieved automatically from Check Point Download Center (current build is $build ). To download the package manually, refer to sk135172. In order to run again without reaching the Check Point Download Center, run $mode process using -skip_upgrade_tools_check flag."
not_available_error_msg="Failed to retrieve the Upgrade Tools package from Check Point Download Center. To download the package manually, refer to sk135172. In order to run again without reaching the Check Point Download Center, run $mode process using -skip_upgrade_tools_check flag."
update_da_message=$'Your Deployment Agent is not up-to-date. To download latest Deployment Agent, refer to sk92449.'
min_da_build=1565

RET_SUCCESS=0
RET_ERROR=1
RET_TIMEOUT=2
max_trials=12


check_da_build_number()
{
	da_build=`da_cli da_status | ${JQ} '."DABuildNumber"' | ${SED} "s/\"//g" 2>/dev/null`
	if [ $? -ne 0 ]; then 
		$ECHO "$internal_error_msg"
        return $RET_ERROR
	fi 
	if [ $da_build -lt $min_da_build ]; then
			$ECHO "$update_da_message"
			return $RET_ERROR
	fi
}

check_for_update_upgrade_tools()
{
	check_da_build_number
	if [ $? -ne 0 ]; then 
		return $RET_ERROR
	fi
	
	trials=1
	check_for_upgrade_tools_update_status
	return $?
}


check_for_upgrade_tools_update_status()
{

	output=`da_cli update_upgrade_tools 2>/dev/null`
	result=`${ECHO} ${output} | ${JQ}  '."Action ID"' | sed 's/"//g'  2>/dev/null`
	
	if [ $result = "0" ]; then
		output=`da_cli upgrade_tools_update_status 2>/dev/null`
		msg=`${ECHO} ${output}  | ${JQ}  '."Message"' 2>/dev/null`
		
		ret_val=0
		case $msg in
			 "\"up to date\"" )
				ret_val=$RET_SUCCESS
			 ;;

			 "\"in progress\"" )
				in_progress_upgrade_tools_update_status
				ret_val=$?
			 ;;

			 "\"outdated\"" )
				$ECHO "$outdated_error_msg"
				 ret_val=$RET_ERROR
			 ;;
			 "\"not available\"" )
				ret_val=$RET_SUCCESS
			 ;;
			 * )
			   ret_val=$RET_SUCCESS

			 ;;
		esac	
	else
        ret_val=$RET_SUCCESS
	fi	
    return $ret_val
}


in_progress_upgrade_tools_update_status()
{
    if [ $trials -ge $max_trials ]; then
        $ECHO "$timeout_error_msg"
        return $RET_TIMEOUT
    fi
    sleep 5
    ((trials++))
    check_for_upgrade_tools_update_status
    return $?
}


#############
check_for_update_upgrade_tools || exit 1