#!/bin/sh

set_vars()
{
	TAR=/usr/bin/tar
	debug_logs_location=$1
	has_failed=$2
}

collect_java_cores()
{
  CORES_DIR=$MDS_FWDIR/log/cores
  coresnames=`ls /var/log/dump/usermode/CPM.*.tar.gz 1 2>/dev/null`
  mkdir -p $CORES_DIR

  for core in $coresnames
  do
    tar -xzf $core -C $CORES_DIR --wildcards --no-anchored 'javacore.*.txt' 1 2>/dev/null
  done
}

collect_sic_local_cert()
{
   $CPDIR/scripts/check_sic_local_cert_problem.sh > /var/log/check_sic_local_cert_output.txt
}

copy_cpdiag_data()
{
   cpdiag_file_name=$1

    if [ -e /.$MDS_FWDIR/cpdiag/$cpdiag_file_name ]; then
      if [ -e /var/log/$cpdiag_file_name ]; then
        cpdiag_tags=`grep "<cpdiag>" /var/log/$cpdiag_file_name | wc -l`
        if [ $cpdiag_tags -ge 2 ]; then
          mv /.$MDS_FWDIR/cpdiag/$cpdiag_file_name /var/log
        else
          tail -n 1 /var/log/$cpdiag_file_name | wc -c | xargs -I {} truncate /var/log/$cpdiag_file_name -s -{}
          tail -n +3 /.$MDS_FWDIR/cpdiag/$cpdiag_file_name >> /var/log/$cpdiag_file_name
          rm /.$MDS_FWDIR/cpdiag/$cpdiag_file_name
        fi
      else
        mv /.$MDS_FWDIR/cpdiag/$cpdiag_file_name /var/log
      fi
    fi
}

collect_logs()
{
	collect_sic_local_cert
	collect_java_cores
	command="$TAR zcvf $debug_logs_location $MDS_FWDIR/log/cpm*elg* $MDS_FWDIR/log/api*elg* /var/tmp/logs_from_peers/* $(realpath $MDS_FWDIR/log/postgres.elg) $CORES_DIR/* /var/log/check_sic_local_cert_output.txt $CPDIR/conf/sic_local_cert.p12 /usr/local/ssl/openssl.cnf $CPDIR/conf/openssl.cnf $CPDIR/log/cpwd.elg*"
	additional_files_for_full_migrate="/opt/CPInstLog/DeploymentAgent* $MDS_FWDIR/log/upgrade_* /var/log/upgrade_stats.xml /var/log/install_Major*.log /opt/CPInstLog/install_Major*.log $MDS_FWDIR/log/verification_report.elg* /var/log/flags_file.json $MDS_FWDIR/log/api.csv $CPDIR/registry/* $UEPMDIR/logs/* /opt/CPInstLog/upgradeDB*.elg"

	if [ X$MDSDIR != "X" ]; then
		command="$command $additional_files_for_full_migrate $MDS_FWDIR/log/mds.elg* $MDSDIR/customers/*/CPshrd-R*/log/migrate* $MDSDIR/customers/*/CPsuite-R*/fw1/log/fwm.elg* $MDSDIR/log/mdsadd_customer_*.elg"
	else
		command="$command $additional_files_for_full_migrate $FWDIR/log/fwm.elg* $CPDIR/log/migrate*"
	fi
	eval $command 1>/dev/null 2>&1
	rm -rf /var/tmp/logs_from_peers
	rm -rf $CORES_DIR
}

collect_server_status()
{
  current_status_log_file="$MDS_FWDIR/log/upgrade_log_current_status.elg"
  path=$(dirname "$0")
  
  if [ -e $path/current_status.sh ]; then
    $(timeout 300 $path/current_status.sh -lf $current_status_log_file -fc) >> $current_status_log_file 2>&1
  elif [ -e $NGM_UPGRADE_TOOLS_PATH/scripts/current_status.sh ]; then
    echo "Running 'current_status.sh' script from tools" >> $current_status_log_file
    $(timeout 300 $NGM_UPGRADE_TOOLS_PATH/scripts/current_status.sh -lf $current_status_log_file -fc) >> $current_status_log_file 2>&1
  else
    echo -e "The script '$NGM_UPGRADE_TOOLS_PATH/scripts/current_status.sh' wasn't found \n $(date) : Running 'df -h':" >> $current_status_log_file
    df -h >> $current_status_log_file  2>&1
  fi
}

upload_advanced_log()
{
  if [ "X$has_failed" != "X0" -a "X$has_failed" != "X" -a "X$UPGRADE_CONTEXT" != "Xcpuse" ]; then
    mac_stamp=`sed -n '2p' /var/log/machine-id | sed 's/://g' `
    time_stamp=`/bin/date '+%e%b%Y-%H%M%S' | /bin/sed 's/ //g'`
    newName=$(dirname $debug_logs_location)/Adv-$mac_stamp-$time_stamp-$(basename $debug_logs_location)
    targetDirectory="CPUSE"

    cp $debug_logs_location $newName
    cpinfo -ndi -r $targetDirectory -f $newName 1>/dev/null 2>&1
    rm $newName
  fi
}

#############################################################################################
set_vars $1 $2
#move /verify_stats.xml - xml file with data for CpDaig- to /var/log
copy_cpdiag_data "verify_stats.xml"
if [ "X$2" == "X-slim_mode" ]; then
  exit 0
fi
#move /upgrade_stats.xml - xml file with data for CpDaig- to /var/log
copy_cpdiag_data "upgrade_stats.xml"
collect_server_status $0
collect_logs
upload_advanced_log
exit 0

