#!/bin/sh

LOG4J2_CONF_FILE=$MDS_FWDIR/conf/log4j2.xml

if [ "X$MDSDIR" = "X" ]; then
    readonly CPM_DIR=$FWDIR/cpm-server
    source $CPDIR/tmp/.CPprofile.sh
else
    readonly FWDIR=$MDS_TEMPLATE
    readonly CPM_DIR=$MDS_TEMPLATE/cpm-server
fi

# check if we are running in demo mode
export DEMO_MODE=`grep -io "cpmp-mgmt-demo" $CPDIR/conf/cp.license | tr '[:upper:]' '[:lower:]'`
if [ "$DEMO_MODE" = "cpmp-mgmt-demo" ]; then
    NGM_DEMO_MODE="demo_setup=TRUE"
fi
# end check if we are in demo mode

if [ "$1" = "-s" -a "$2" = "-d" -o ! -z "$NGM_SERVER_UPGRADE" ]; then
    NGM_UPGRADE_MODE="upgrade_setup=TRUE"
else
	NGM_UPGRADE_MODE="upgrade_setup=FALSE"
fi

#Not Gaia! - we should increase fd max to 65535
if [ ! -f /etc/appliance_config.xml ]
then
    ulimit -n 65535
    OUT=$?
    echo $OUT
    if [ ! $OUT -eq 0 ];then
      echo "FATAL ERROR - Failed to increase FD to 65535 - server will shut down please call support"
    fi
fi

$CPDIR/bin/CPSetupInfo $CPDIR/conf/CpSetupInfo_setups.conf $CPDIR/conf/CpSetupInfo_resourceProfiles.conf $CPDIR/conf/CPSetupInfo.props $FWDIR/conf/cpmServerSettings.props $NGM_DEMO_MODE $NGM_UPGRADE_MODE
# We export all variables from serverSettings.props so cpm process will be able to set configuration upon them.
sh $MDS_FWDIR/scripts/apply_override_server_settings.sh $MDS_FWDIR/conf/cpmServerSettings.props
eval $(cat $MDS_FWDIR/conf/cpmServerSettings.props | sed 's/ #override//' | sed 's/^/export /')
eval "$(cat $MDS_FWDIR/conf/cpmEnvVars.conf)"

# Setup SMC-Backup const parameters
IS_SMC_BACKUP=`$CPDIR/bin/cpprod_util CPPROD_GetValue "fw1" "SMCBackup" 1`
if [ "$IS_SMC_BACKUP" = "1" ]; then
    export SMC_SETUP_DOMAINID=`$CPDIR/bin/cpprod_util CPPROD_GetValue "fw1" "DomainUID" 1`
    export SMC_SETUP_DOMAINNAME=`$CPDIR/bin/cpprod_util CPPROD_GetValue "fw1" "DomainName" 1`
fi

CPM_CACHE_NAME="cpm_server_java_cache"
CPM_CACHE_OPTS="none"

if [ "$JVM" = "64" ]; then
    #export this for Solr to run on the same JVM. Please don't remove without changing SolrServerRunner and solr_start.sh
#    export JVM
    readonly JAVA_HOME=$JAVA_HOME_64
    CPM_CACHE_NAME=${CPM_CACHE_NAME}"_64"
fi

readonly CPM_LOG=$FWDIR/log/cpm.stdout.elg
readonly JAVA_CODE_CACHE="$CPM_DIR/code-cache"
readonly JAVA_TMP_DIR="$FWDIR/tmp"
readonly CPM_DUMP_DIR=/var/log/dump/usermode


# make sure java code cache folder exists
if [ ! -d $JAVA_CODE_CACHE ]; then
    mkdir $JAVA_CODE_CACHE
fi

cpmPid=$(ps -efww | grep java | grep cpm.Cpm | awk '{print $2}')
if [ -n "$cpmPid" -a "$1" == "-s" ]; then
    echo "Another CPM server is running (pid $cpmPid), close it first before running another one"
    exit -1
fi

if [[ -n "$NGM_CPM_USE_CODE_CASH" && $NGM_CPM_USE_CODE_CASH -eq 1 ]]; then
    cache_state_file=$JAVA_TMP_DIR/stateFile
    if [ "$1" == "-s"  -a "$(ls -A $JAVA_CODE_CACHE)" ]; then
        for cache in $CPM_CACHE_NAME; do
            $JAVA_HOME/bin/java -Xshareclasses:cacheDir=$JAVA_CODE_CACHE,name=$cache,printStats > $cache_state_file 2>&1
            grep -q "Cache is accessible to current user" $cache_state_file
            if [ $? -eq 1 ]; then
                echo "Cache $cache is corrupted. Deleting it..."
                $JAVA_HOME/bin/java -Xshareclasses:cacheDir=$JAVA_CODE_CACHE,name=$cache,destroy >> $CPM_LOG 2>&1
            fi
            rm $cache_state_file
        done
    fi

    CPM_CACHE_OPTS="cacheDir=$JAVA_CODE_CACHE,name=$CPM_CACHE_NAME -Xscmx300m"
fi

if [ "$1" == "no_sic_cert" ]; then
   shift
fi

if [ "$1" == "-s" -a "$2" == "-d" ]; then
    $CPDIR/bin/cpprod_util CPPROD_DelValue FW1 NeverSynced 1 >> $CPM_LOG 2>&1
fi


export TMPDIR=/var/log/tmp

# default settings for jvm memory allocation.
# JAVA 7
#if [ -z "$NGM_JAVA_OPTS" ]; then
#    NGM_JAVA_OPTS="-Xmx$NGM_CPM_MAX_HEAP -Xms$NGM_CPM_MIN_HEAP $NGM_HEAP_POLICY -Djava.io.tmpdir=$JAVA_TMP_DIR -Xaggressive -Xshareclasses:cacheDir=$JAVA_CODE_CACHE,name=cpm_server_java_cache"
#    NGM_JAVA_DUMP_OPTS=(-Xdump:directory=${CPM_DUMP_DIR} -Xdump:heap:events=gpf+user -Xdump:system:none -Xdump:system:events=gpf+abort+traceassert+corruptcache -Xdump:tool:events=gpf+user+abort+traceassert+corruptcache,priority=2,range=1..0,exec="/etc/coredump/javaCompress.sh  $CPM_DUMP_DIR" -Xdump:tool:events=systhrow,filter=java/lang/OutOfMemoryError,priority=2,range=1..0,exec="/etc/coredump/javaCompress.sh $CPM_DUMP_DIR" -Xdump:tool:events=throw,filter=java/lang/OutOfMemoryError,priority=1,exec="kill -9 %pid")
#fi

# JAVA 8
if [ -z "$NGM_JAVA_OPTS" ]; then
    NGM_JAVA_OPTS="-Xaot:forceaot -Xmx$NGM_CPM_MAX_HEAP -Xms$NGM_CPM_MIN_HEAP $NGM_HEAP_POLICY -Dlog4j2.configurationFile=$LOG4J2_CONF_FILE -Djava.io.tmpdir=$JAVA_TMP_DIR -Xaggressive -Xshareclasses:$CPM_CACHE_OPTS"
    NGM_JAVA_DUMP_OPTS=(-Xdump:directory=${CPM_DUMP_DIR}
                        -Xdump:heap:events=gpf+user
                        -Xdump:tool:none
                        -Xdump:tool:events=user,priority=1,range=1..0,exec="javaCompress.sh CPMUSER %pid"
                        -Xdump:tool:events=gpf+abort+traceassert+corruptcache,priority=1,range=1..0,exec="javaCompress.sh CPM %pid"
                        -Xdump:tool:events=systhrow,filter=java/lang/OutOfMemoryError,priority=1,range=1..0,exec="javaCompress.sh CPM %pid"
                        -Xdump:tool:events=throw,filter=java/lang/OutOfMemoryError,priority=1,exec="kill -9 %pid")
fi

# wrapped the remove operation to run just in case we are starting the server and not on every cpm command
if [ "$1" == "-s" ]; then
    rm -f $MDS_FWDIR/tmp/cpm.pid
    rm -f $MDS_FWDIR/tmp/cpm_failed_start
fi

if [[ -n "$CPM_PROFILING" && $CPM_PROFILING -eq 1 ]]; then
    NGM_JAVA_OPTS="$NGM_JAVA_OPTS -Xhealthcenter:transport=jrmp"
fi

if [[ -n "$JPROFILER_DIR" ]]; then
    JPROFILER_ARCH="x86"
    if [ "$JVM" = "64" ]; then
        JPROFILER_ARCH="x64"
    fi
    
    if [[ -n "$OFFLINE_PROFILING" ]]; then
        NGM_JAVA_OPTS="$NGM_JAVA_OPTS -Xshareclasses:none -agentpath:${JPROFILER_DIR}/bin/linux-$JPROFILER_ARCH/libjprofilerti.so=offline,id=107,config=${JPROFILER_DIR}/jprofiler_config.xml -Djprofiler.jmxServerPort=5566 -Djprofiler.mBeanDelay=30"
    else
        NGM_JAVA_OPTS="$NGM_JAVA_OPTS -Xshareclasses:none -agentpath:${JPROFILER_DIR}/bin/linux-$JPROFILER_ARCH/libjprofilerti.so=port=8849"
    fi
fi

# CR02342618 & CR02355803 - WA (Pending IBM) - Problem with kerberos AES encryption for tgt(config file limit to default_tgs_enctypes = rc4-hmac only)
readonly CPM_KRB5_CONF_FILE=$MDS_FWDIR/conf/krb5.conf
if [ -f $CPM_KRB5_CONF_FILE ]; then
    NGM_JAVA_OPTS="$NGM_JAVA_OPTS -Djava.security.krb5.conf=$CPM_KRB5_CONF_FILE"
fi

cd $CPM_DIR
# CR01925959 - temp flag to catch NPE from IdentityHashMap
echo "cpm.sh exec line: $JAVA_HOME/bin/java -D_CPM=TRUE $NGM_JAVA_OPTS \"${NGM_JAVA_DUMP_OPTS[@]}\" -Dfile.encoding=\"UTF-8\" -cp \"*:\" \"com.checkpoint.management.cpm.Cpm\" \"$@\" " >> $CPM_LOG
# JAVA 7
#exec $JAVA_HOME/bin/java -D_CPM=TRUE  $JIT_DEBUG_OPTS -Xdump:java+system:events=throw,filter=java/lang/NullPointerException#java/util/IdentityHashMap.put $NGM_JAVA_OPTS "${NGM_JAVA_DUMP_OPTS[@]}" -Dfile.encoding="UTF-8" -cp "*" "com.checkpoint.management.cpm.Cpm" "$@" >& $CPM_LOG

# JAVA 8
NGM_JAVA_OPTS="$NGM_JAVA_OPTS -Xjit:exclude={com/checkpoint/management/dleserver/coresvc/internal/SchemaMgrSvcImpl.getClassInfo*},exclude={com/checkpoint/management/object_store/ObjectStoreSessionImpl.findFieldsBySearchQueryEx*},exclude={org/eclipse/persistence/mappings/AggregateCollectionMapping.compareForChange*}"

NGM_CLASSPATH_OPT="*:"
if [[ -n "$NGM_UPGRADE_TOOLS_JARS_PATH" ]]; then
    NGM_CLASSPATH_OPT="$NGM_UPGRADE_TOOLS_JARS_PATH/*:$NGM_CLASSPATH_OPT"
fi


exec $JAVA_HOME/bin/java -D_CPM=TRUE $NGM_JAVA_OPTS "${NGM_JAVA_DUMP_OPTS[@]}" -Dfile.encoding="UTF-8" -cp $NGM_CLASSPATH_OPT "com.checkpoint.management.cpm.Cpm" "$@" >& $CPM_LOG

# DON'T PUT ANYTHING HERE - IT WON'T BE EXECUTED!
