#!/bin/sh 

set_vars()
{
    log_file=$MDS_FWDIR/log/cpm_overall_checks.elg
    fast=0
    i=0
    currentStatusExitCode=0
}

parse_args()
{
    while [ $# -gt 0 ]
    do
        case $1 in
            -fc|--fast_check)
                fast=1
                ;;
            -lf|--log_file)
                if [ $# -gt 0 ]; then
                    shift
                    log_file=$1
                else
                    echo "log file path is missing"
                    exit 1
                fi
                ;;
            *)
                echo "unrecognized args. do nothing. exit."
                ;;
        esac
        shift
    done
}

print_into_file()
{
    current_date=""
    if [ "X$2" = "X1" ]; then
        current_date="$(date):: "
    fi
    echo "${current_date}$1" >> $log_file       2>&1
}

check_is_primary()
{
	IS_PRIMARY=0
    isPrimary=`$CPDIR/bin/cpprod_util FwIsPrimary| /bin/sed 's/ //g'`;
    if [ "X1" = "X$isPrimary" ]; then
        IS_PRIMARY=1
    fi
}

check_is_mlm()
{
    IS_MLM=0
    if [ "X$MDSDIR" != "X" ]; then
        isMLM=`$CPDIR/bin/cpprod_util CPPROD_GetValue PROVIDER-1 LoginServer 1`
        if [ "X0" = "X$isMLM" ]; then
            IS_MLM=1
        fi
    fi
}

check_is_log_server()
{
	LOG_SERVER=0
    if [ "X$MDSDIR" = "X" ]; then
        isLogServer=`$CPDIR/bin/cpprod_util FwIsLogServer| /bin/sed 's/ //g'`
        if [ "X1" = "X$isLogServer" ]; then
            LOG_SERVER=1
        fi
    fi
}

check_is_mds()
{
    MDS=1
    if [ "X$MDSDIR" = "X" ]; then
        MDS=0
    fi
}

print_cpm_status()
{
    check_is_mds
    if [ $MDS -eq 1 ]; then
        print_into_file "mdsstat:" 1
        mdsstat >> $log_file            2>&1
    else
        print_into_file "cpwd_admin list:" 1
        cpwd_admin list >> $log_file    2>&1
    fi
    print_into_file "--> The PID of the cpm is $(ps -efww | grep java | grep cpm.Cpm | awk '{print $2}')" 0
}

print_postgres_status()
{
    print_into_file "The postgres status is:" 1
    su -s /bin/bash -c "$MDS_FWDIR/scripts/postgres_commands.sh status" cp_postgres >> $log_file 2>&1
    psg_st=$?
    number_of_rows=5
    if [ $psg_st -eq 0 ]; then
        print_into_file "tasknotification table, showing the recent $number_of_rows rows" 0
        psql_client cpm postgres -c "select taskid,progresspercentage, status, creationtime, lastmodifytime from tasknotification order by creationtime DESC limit $number_of_rows" >> $log_file 2>&1
    else
        print_into_file "The postgres status is $psg_st" 1
    fi
}

print_machine_type()
{
    check_is_mds
    check_is_primary
    check_is_mlm
    check_is_log_server

    print_into_file "is mds: $MDS ; is primary: $IS_PRIMARY ; is mlm: $IS_MLM ; is log server: $LOG_SERVER" 0
}

print_connectivity()
{
    if [ "X$fast" != "X1" ]; then
        psql_client cpm postgres -t -A -c "select ipaddress4 from CPnetworkobject_data where dlesession=0 AND not deleted order by lastmodifytime DESC" | while read ip; do
            res=`ping $ip -c 2 | tail -n 2`
            print_into_file "$ip: $res" 0
        done
    fi
}

process_info()
{
    print_into_file "$((++i)). Process info:" 1
    print_machine_type
    print_cpm_status
    print_postgres_status
    print_into_file "$mode process context is: $UPGRADE_CONTEXT" 0
    print_into_file "FWDIR is $FWDIR" 0
    echo -e "\n" >> $log_file
}

system_info()
{
    print_into_file "$((++i)). System info:" 1
    print_into_file "The bash version is ${BASH_VERSION}" 0
    head -n 1 $MDS_FWDIR/conf/cpmServerSettings.props >> $log_file 2>&1
    grep 'MAX' $MDS_FWDIR/conf/cpmServerSettings.props | awk -v OFS="\t" '{print $5, $1}' >> $log_file  2>&1
    echo -e "\n" >> $log_file
    api status >> $log_file  2>&1
    currentStatusExitCode=$?
    echo "'api status' exit code is $currentStatusExitCode" >> $log_file  2>&1
    grep Mem /proc/meminfo >> $log_file  2>&1
    df -h >> $log_file  2>&1
    echo -e "\nCPLIC:" >> $log_file
    cplic print >> $log_file  2>&1
    echo -e "\nPostgres log:" >> $log_file
    ls -ltr /var/log/pg_log >> $log_file  2>&1
     echo -e "\n" >> $log_file
}

environment_info()
{
    print_into_file "$((++i)). Environment info:" 1
    psql_client cpm postgres -c "select name,ipaddress4,icon,active,sicname from CPnetworkobject_data where dlesession=0 AND not deleted order by lastmodifytime DESC" >> $log_file 2>&1
    psql_client cpm postgres -c "select displayname,creationtime,lastmodifytime from tasknotification where comments='Synchronized successfully' order by lastmodifytime DESC limit 10" >> $log_file 2>&1
    print_connectivity
    echo -e "\n" >> $log_file
}

######################################################################################################

set_vars 
parse_args $@
print_into_file ">>>>>>>>>>>>>>>>>>>>> Starting with status checking $(date) <<<<<<<<<<<<<<<<<<<<< " 0
process_info
system_info
environment_info
print_into_file ">>>>>>>>>>>>>>>>>>>>>>> Done with status checking $(date) <<<<<<<<<<<<<<<<<<<<<<< " 0
exit $currentStatusExitCode
