#!/bin/sh

. /opt/CPshared/5.0/tmp/.CPprofile.sh >> /dev/null

# run cpd_config -p and print the AddOn that are not enabled

remove_double_quotes() { sed 's/"//g'; }

remove_spaces_between_at_ends_and_around_colon()
{
    # find spaces at: line start, before colonm after colon and at the line end
    # add all otheer values into a group
    # convert the input into group1 colon group2

    sed -r 's/\s*([a-zA-Z].*)\s*:\s*(.*[a-zA-Z0-9_-])\s*$/\1:\2/'
}

add_doublequotes_around_name_value_pairs() { sed -r 's/([^:]*):(.*)/"\1": "\2"/'; }

add_comma_separator_at_end_of_name_value_pairs() { sed -r 's/(".*": ".*")/\1,/'; }

create_objects_at_AddOn_delimiter() { sed -r 's/"AddOn.*/}, { \0/' ; }

remove_newlines() { tr -d '\n'; }

remove_comma_before_brace() { sed 's/,\}/}/g'; }

add_json_list_open_bracket() { echo '['; }
add_json_list_closed_bracket() { echo ']'; }

add_json_list_open_brace() { echo '{'; }
add_json_list_closed_brace() { echo '}'; }

remove_dummy_json_object() { jq -r  '.[1:]'; }

select_disabled_addon() { jq -r '.[] | select(.Enabled != "yes") | .AddOn'; }

select_disabled_addon() { jq -r '.[] | select(.Enabled != "yes") | .AddOn'; }

join() { local connectionStr="$1"; tr '\n' $connectionStr | sed "s/$connectionStr$/\n/"; }

domain_check()
{
    # braces combine the output of all command into a single stdout stream
    (
        # add the start of a list of json objects: [{}, {...}, {...}
        # each object begins with the word "AddOn"

        # the first object is a dummy object because AddOn separates between elements with "}, {" i
        # and we want valid json. It will be handled later.

        add_json_list_open_bracket; add_json_list_open_brace;

        # run cpd_config  and normalize the output
        cpd_config -p  |
            # there are unnecessary double quotes in the output
            remove_double_quotes |

            # all fields have "name: value" form. Remove spaces.
            remove_spaces_between_at_ends_and_around_colon |

            # put doublequoes around name value fields as expected by json
            add_doublequotes_around_name_value_pairs |

            # put "}, {" before each AddOn term. AddOn is the delimiter for each new entry
            create_objects_at_AddOn_delimiter |

            # every name value pair needs a comma to seaparte them, except the last
            add_comma_separator_at_end_of_name_value_pairs

        # close the last object and the list
        add_json_list_closed_brace; add_json_list_closed_bracket

    ) |
    # remove newlines to enable regex matching across lines
    remove_newlines |

    # remove final comma before the object close brace because json is not forgiving
    remove_comma_before_brace |

    # Now we have valid json output. Remove the first dummy object in the list.
    remove_dummy_json_object |

    # select the objectds from the list whose Enabled field is not "yes"
    # output the AddOn field's value for those objects
    select_disabled_addon |

    #Separate the results with commas
    join ,

}

run(){
  message=""
  if [ "$MDSDIR" != "" ]; then
     res=""
	   #$MDSVERUTIL AllCMAs | while read cma;
	   for cma in `$MDSVERUTIL AllCMAs`
	   do
	   	# Defines the current customar/domain as the working environment.
	   	mdsenv $cma

      result="$(domain_check)"
      if [[ ! -z "$result" ]];then
          res+=" In the Domain Server "$cma" the disabled Add ons are:"$result"-"
      fi
	   done
	   # Returns to the global working environment.
	   mdsenv
	   if [[ ! -z "$res" ]];then
	      message="Found disabled addons in some Domain Servers -"
	      message+=$res
	   fi
  else
     result="$(domain_check)"
     if [[ ! -z "$result" ]];then
          message+="Found disabled LSM Add ons: -"
          message+="The Add ons are:"$result"-"
     fi
  fi
}

parse_args $@

# Runs over all the domains
run     2>/dev/null
printf "$message"
