#!/bin/bash

FLAGS_FILE_CONS="/var/log/upgrade_flags_file.json"
FLAGS_FILE_REL="/var/log/upgrade_flags_file_reload.json"
TEMP_FILE="/var/log/upgrade_flags_file_temp.json"
NUMBER_OF_ARGS=$#
OPTION=$1
PROPERTY=$2
DELETED="\"deleted\""
NOT_FOUND_CODE=2

function print_usage(){
	echo "Usages:"
	echo "-s(i) | -set(i)	> setting flag (immediately)   : modify_upgrade_flags.sh -s(i) <PROPERTY> [<VALUE>]"
	echo "-u(i) | -unset(i)	> unsetting flag (immediately) : modify_upgrade_flags.sh -u(i) <PROPERTY>"
	echo "-a(i) | -add(i) 	> adding values to existed flag : modify_upgrade_flags.sh -a(i) <PROPERTY> [<VALUE>]"
	echo "* In case the <VALUE> is not given, the default value will be 'true'"
	echo "** For enabling setting flags immediately (while the upgrade process is ongoing), run: modify_upgrade_flags.sh -s enable_reload_immediate_file"
	echo ""
	echo "-print | -p | -show > printing the flags files : modify_upgrade_flags.sh -p"
	echo "-get | -g > getting flag : modify_upgrade_flags.sh -g <PROPERTY> (returns 'null' if doesn't exist)"
	echo "-remove | -rm > removing flag files: modify_upgrade_flags.sh -remove [reload_file_only | rfo] (removes the whole file(s))"
	echo "-load_from_file | -lff > loading flags from a file : modify_upgrade_flags.sh -lff <file_path> (appends the new flags to the existed ones)"
	echo "* The loading file should have each flag in separated line, a line is of the form: <Property : \"val_1\",\"val_2\",\"val_3\">"
}

function add_flag(){
	FLAGS_FILE=$1
	PAIR="{\"$2\":[$3]}"
	
	if [ ! -e $FLAGS_FILE -o ! -s $FLAGS_FILE ];
		then
			echo $PAIR | jq '.' > $TEMP_FILE
			check_result_jq $? $PAIR "added"
		else
			jq -r 'if .'$2' ? then .'$2'='"[$3]"' else .+ {"'"$2"'":'"[$3]"'} end' $FLAGS_FILE > $TEMP_FILE
			check_result_jq $? $PAIR "added to"
	fi	
}

function remove_flag(){
	FLAGS_FILE=$1
	
	if [ -e $FLAGS_FILE -o -s $FLAGS_FILE ];
		then
			jq --arg PROPERTY "$2" 'del(.'$2')' $FLAGS_FILE > $TEMP_FILE 
			check_result_jq $? $2 "removed from"
		else
			echo "$FLAGS_FILE is empty - do nothing.."
	fi
}

function get_flag(){
	if [ ! -e $1 ]; then
		echo "$1 doesn't exist"
		exit $NOT_FOUND_CODE
	fi
	res=$(jq -r '.'$2'?' $1)
	echo "$res"
	if [ "$res" = "null" ]; then
		exit $NOT_FOUND_CODE 
	fi
}

function check_result_jq(){
	if [ $1 -ne 0 ]; 
		then
			echo "A flag was NOT $3 $FLAGS_FILE, check your input: $2"
			rm $TEMP_FILE
		else
			mv $TEMP_FILE $FLAGS_FILE 
			echo -e "A flag was $3 $FLAGS_FILE: \n $2"
	fi
}

function insert_list(){
	VALUE=""
	i=0
	echo "Please insert values to the list, press 'enter' to continue"
	read -p "value $((++i)): " from_user 
	if [ "$from_user" != "" ]; 
		then
			VALUE="\"$from_user\""
		else
			echo "No value was given. exiting.."
			exit 1
	fi
	
	read_from_user
}

function read_from_user(){
	read -p "value $((++i)): " from_user
	if [ "$from_user" != "" ]; then
		VALUE="$VALUE,\"$from_user\""
		read_from_user
	fi
}

function print_files(){
	if [ -e $1 ]; 
		then
			echo -e "The file $1:"
			cat $1
			echo ""
		elif [ "$2" = "-d" ];
    		then
			echo -e "The file $1 doesn't exist.\n"
	fi
}

function read_from_external_file(){
	if [ ! -e $1 ]; then
		echo "$1 doesn't exist"
		print_usage
		exit 1
	fi

	while read -r line;
	do
		line=$(echo $line | sed 's/:/ /')
		PROP=$(echo $line | awk '{print $1}')
		VAL=$(echo $line | awk '{$1= ""; print $0}' | sed 's/ //g')
		if [ "X$VAL" = "X" ]; then
    		VAL="\"true\""
    fi
		add_flag $FLAGS_FILE_CONS $PROP $VAL
	done < $1
}

function add_to_flag(){
  if [ -e $1 ]; then
    current=$(jq -r '.'$2'?' $1)
    if [ "X$current" = "Xnull" ] || [ "X$current" = "X[]" ]; then
      add_flag $1 $2 $3
    else
      new_value=$(echo $current | sed 's/ //g' | sed 's/\]/,'$3'/' | sed 's/\[//')
      add_flag $1 $2 $new_value
    fi
  else
    add_flag $1 $2 $3
  fi
}

function check_args(){
	if [ $NUMBER_OF_ARGS -lt $1 ]; then
		echo "Incorrect number of arguments"
		print_usage
		exit 1
	fi
}

check_args 1

if [ $NUMBER_OF_ARGS -lt 3 ];
	then
		VALUE="\"true\""
	elif [ $3 = "-insert_list" -o $3 = "-il" ]; then
		insert_list
	else
		VALUE="\"$3\""
fi	


case $OPTION in
	-s | -set)
		check_args 2
		add_flag $FLAGS_FILE_CONS $PROPERTY $VALUE
		;;
	-si | -seti)
		check_args 2
		add_flag $FLAGS_FILE_CONS $PROPERTY $VALUE
		add_flag $FLAGS_FILE_REL $PROPERTY $VALUE
		;;
	-u | -unset)
		check_args 2
		remove_flag $FLAGS_FILE_CONS $PROPERTY
		;;
	-ui | -unseti)
		check_args 2
		remove_flag $FLAGS_FILE_CONS $PROPERTY
		add_flag $FLAGS_FILE_REL $PROPERTY $DELETED
		;;
  -a | -add)
    check_args 2
    add_to_flag $FLAGS_FILE_CONS $PROPERTY $VALUE
    ;;
  -ai | -addi)
    check_args 2
    add_to_flag $FLAGS_FILE_CONS $PROPERTY $VALUE
    add_flag $FLAGS_FILE_REL $PROPERTY $new_value
    ;;
	-p | -print | -show)
		print_files $FLAGS_FILE_CONS -d
    print_files $FLAGS_FILE_REL $PROPERTY
		;;
	-g | -get)
		check_args 2
		get_flag $FLAGS_FILE_CONS $2
		;;
	-load_from_file | -lff)
		check_args 2
		read_from_external_file $2
		;;
	-rm | -remove)
		if [ "X$2" != "Xreload_file_only" -a "X$2" != "Xrfo" ]; then
			rm $FLAGS_FILE_CONS 2>/dev/null
		fi
		rm $FLAGS_FILE_REL 	2>/dev/null
		;;
  -h | -help)
    print_usage
    exit 0
    ;;
	*)
		echo $OPTION is not a valid option:
		print_usage
		exit 1
esac

exit 0