#!/bin/bash

source /opt/CPshared/5.0/tmp/.CPprofile.sh &> /dev/null;

CPPROD_UTIL=$CPDIR/bin/cpprod_util
SED="/bin/sed"
SU="/bin/su" 
PWD="/bin/pwd"
MAX_TRIES=60
SLEEP="/bin/sleep"
CAT="/bin/cat"
ECHO="/bin/echo"
CHMOD="/bin/chmod"
AWK=/bin/awk
ECHO="/bin/echo"
GREP="/bin/grep"
RM="/bin/rm -f"
MV="/bin/mv"
SUCCESS_MSG="success"
SKIP_NEWUPG_PUV_FLAG_FILE="/var/tmp/skip_new_upgrade_puv"
SKIP_PRIMARY_CHECK_FLAG_FILE="/var/tmp/do_not_wait_for_primary_upgrade"
CHANGED_IP_FILE="/var/log/mdss.json"
ALLOW_R80_20_UPGRADE_FLAG_FILE="/var/tmp/allow_r80_30_new_upgrade"
LOG_FILE=$MDS_FWDIR/log/verification_report.elg
LOG_FILE_OLD=$MDS_FWDIR/log/verification_report.elg.old


unset NGM_UPGRADE_TOOLS_JARS_PATH

check_valid_license()
{
	rc=0
	if [ -z $MDSDIR ]; then
		cplic check mgmtcore >> $LOG_FILE 2>&1
		rc=$(($rc+$?))
		cplic check policyui >> $LOG_FILE 2>&1
		rc=$(($rc+$?))
		cplic check clm >> $LOG_FILE 2>&1
		rc=$(($rc+$?))		
	else
		cplic check gblp >> $LOG_FILE 2>&1
		rc=$(($rc+$?))
		cplic check mlogs >> $LOG_FILE 2>&1
		rc=$(($rc+$?))
		cplic check mspopt >> $LOG_FILE 2>&1
		rc=$(($rc+$?))
		cplic check mdclm >> $LOG_FILE 2>&1
        rc=$(($rc+$?))

	fi
	if [ $rc -lt 1 ]; then
		return 1
	else 
		return 0
	fi
}

check_postgres_status()
{
  local MAX_TRIES=3
	local RET=1
	local RETURN_VAL=$1
  PREV_LOCATION=`pwd`
	cd $PGDIR

	if [ -f /var/tmp/postgres_max_tries ];then
		MAX_TRIES=`$CAT /var/tmp/postgres_max_tries`
	fi

	$SU -s /bin/bash -c "FWDIR=$MDS_FWDIR $MDS_FWDIR/scripts/postgres_commands.sh status" cp_postgres >> $LOG_FILE 2>&1
	RET=$?

	if [ $RET -eq 0 ];then
	  $ECHO "waiting for postgres to accept connections" >> $LOG_FILE 2>&1
		RET=3
		TRIES=0
		while [ $RET -ne 0 -a $TRIES -lt $MAX_TRIES ]
		do
		  $MDS_FWDIR/bin/psql_client -U postgres cpm -c "select 1;" >> $LOG_FILE 2>&1
			RET=$?
			sleep 1
			let TRIES=TRIES+1
		done
	fi

  $ECHO "check_postgres_status end. return value is $RET" >> $LOG_FILE 2>&1
  cd $PREV_LOCATION
	eval $RETURN_VAL="'$RET'"
}

find_current_version()
{
  ver=`$CPDIR/bin/cpprod_util CPPROD_GetVerText CPshared|sed "s/ //g"`
  releaseVer=`$CPDIR/bin/cpprod_util CPPROD_GetValue FW1 CurrentMgmtReleaseVersion 1 2>/dev/null`
  st=$?
 
  currentVersionName=""
  if [ $st -eq 0 ]; then
	if [ "X$releaseVer" = "XGA" ]; then
		currentVersionName=$ver
	else
		currentVersionName="${ver}.${releaseVer}"
	fi
  else
	if [ "$ver" = "R80.20" ]; then
		currentVersionName="R80.20.M1"
	fi
  fi
  echo $currentVersionName
}

find_upgrade_tools_version()
{
	upgradeToolsVersion=""
	if [ ! -z $NGM_UPGRADE_TOOLS_PATH -a -f $NGM_UPGRADE_TOOLS_PATH/version.info ]; then
		upgradeToolsVersion=`$CAT $NGM_UPGRADE_TOOLS_PATH/version.info | $GREP "UpgradeVersion" | $AWK -F = '{print $2}'`
	fi 
	echo $upgradeToolsVersion
}

busy_wait_check_postgres_status()
{
	local RETURN_VAL=$1
	cd $PGDIR
	TRIES=0
	local POSTGRES_STATUS_RET=1
	local QUERY_RET=1
	if [ -f /var/tmp/postgres_max_tries ];then
		MAX_TRIES=`$CAT /var/tmp/postgres_max_tries`
	fi
	while [ $POSTGRES_STATUS_RET -ne 0 -a $TRIES -lt $MAX_TRIES ]
	do
		check_postgres_status POSTGRES_STATUS_RET
		$SLEEP 1
		let TRIES=TRIES+1
	done
	TRIES=0
	if [ $POSTGRES_STATUS_RET -eq 0 ];then
		TRIES=0
		while [ $QUERY_RET -ne 0 -a $TRIES -lt $MAX_TRIES ]
		do
			$ECHO "==================================================$MDS_FWDIR/bin/psql_client -U postgres cpm -c select 1" >> $LOG_FILE 2>&1
			$MDS_FWDIR/bin/psql_client -U postgres cpm -c "select 1;" >> $LOG_FILE 2>&1
			QUERY_RET=$?
			$SLEEP 1
			let TRIES=TRIES+1
		done
	else
		$ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support." >> $LOG_FILE 2>&1
	fi		
	eval $RETURN_VAL="'$QUERY_RET'"
}


find_active_user_domains()
{
    active=`$CPPROD_UTIL FwIsActiveManagement| $SED 's/ //g'`;

    if [ $? -ne 0 ]; then
        $ECHO "Could not check state of cma $element."  >> $LOG_FILE 2>&1
        $ECHO "Could not run compatibility check. Failed to read 'FwIsActiveManagement' value from the registry. Please contact Check Point Support for further assistance."
        exit 1
    fi

    if [ $active -eq 1 ]; then
       uid=`$CPDIR/bin/cpprod_util CPPROD_GetValue "FW1" DomainUID 1`;
       if [ $? -ne 0 ]; then
          $ECHO "Could not find uid of domain for cma $element."    >> $LOG_FILE 2>&1
          $ECHO "Could not run compatibility check. Failed to read 'DomainUID' value from the registry. Please contact Check Point Support for further assistance."
          exit 1
       fi
       actives+="'"
       actives+="$uid"
       actives+="',"
    fi
}

init_verification_log_file()
{
    if [ -f "$LOG_FILE_OLD" ]; then
        $RM $LOG_FILE_OLD
    fi
	
    if [ -f "$LOG_FILE" ]; then
        $MV $LOG_FILE $LOG_FILE_OLD
    fi
}

exitAndLog() {
	if [ $# -eq 1 ]; then
		$ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
		exit "$1"
	else
		$ECHO "$2"
		exit "$1"
	fi
}


# if skip puv
$MDS_FWDIR/scripts/modify_upgrade_flags.sh -get skip_puvs >> /dev/null
if [ $? -eq 0 ] || [ ! -z "$SKIP_NEWUPG_PUV" ] || [ -e "$SKIP_NEWUPG_PUV_FLAG_FILE" ]; then
		$ECHO $SUCCESS_MSG
		exit 0
fi

#slim mode for PUVs we want to run from upgrade_clish.sh before we start the ImportTask which runs all PUVs
if [ "X$1" == "X-slim_mode" ]; then
  check_valid_license
  if [ $? -ne 0 ]; then
	 output="The server does not have a valid license. For assistance with your license, contact Check Point Account Services: https://help.checkpoint.com."
	 $ECHO $output
	 exit 1
  else
  	 $ECHO $SUCCESS_MSG
	 exit 0
  fi
fi

# mv verification_report.elg if exist to verification_report.elg.old
init_verification_log_file

# Check is MLM or Secondary setup
isManagement=`$CPPROD_UTIL CPPROD_GetValue FW1 FWManagement 4`
isLogServer=`$CPPROD_UTIL CPPROD_GetValue FW1 LogServer 4`
STATUS=0

# Check conditions on allowed setups
if [ "$isManagement" == "1" -o "$isLogServer" == "1" ]; then

  POSTGRES_WAS_DOWN=0
  POSTGRES_STATUS=3
  
  WORK_DIR=`$PWD`
  check_postgres_status POSTGRES_STATUS
  cd $WORK_DIR
  if [ $POSTGRES_STATUS -ne 0 ]; then
	POSTGRES_WAS_DOWN=1
  fi
  if [ $POSTGRES_WAS_DOWN -eq 1 ];then
	  $ECHO "==================================================FWDIR=$MDS_FWDIR $MDS_FWDIR/scripts/postgres_start.sh" >> $LOG_FILE 2>&1
	  FWDIR=$MDS_FWDIR $MDS_FWDIR/scripts/postgres_start.sh  >> $LOG_FILE 2>&1
	  POSTGRES_STATUS=3
	  busy_wait_check_postgres_status POSTGRES_STATUS
	  cd $WORK_DIR
	  if [ $POSTGRES_STATUS -ne 0 ]; then
		  $ECHO "Failed to start. For more information, see $LOG_FILE."	
		  exit 2
	  fi
  fi
  
  i=1  
  output=""
  
  #building a list of active local domains in variable active_cmas
  #all PUVs that that run a query on the database, must run only for the active domains!!!
  actives=""
 if [ ! -z $MDSDIR ]; then
    for  element in `$MDSVERUTIL  AllCMAs`
    do
        mdsenv $element
        find_active_user_domains
    done
	mdsenv
 else
      find_active_user_domains
 fi

  if [ ! -z $actives ]; then
      active_cmas=${actives%?}
  else
      active_cmas=$actives
  fi


  # 1. Checking IPS private package
  ips_private=$(psql_client cpm postgres -t -c "select objid from customipsupdatepackagelist_data where domainid='a0bbbc99-adef-4ef8-bb6d-cebcebcebceb';" 2>/dev/null)
  if [ $? -ne 0 ]; then
	$ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
	exit 3
  fi
  if [ ! -z "$ips_private" ]; then
	  output="${output}\n$i. Operation is not supported with IPS private package. Please revert IPS to the previous version."
	  i=$((i+1))
  fi
  
  # 2. Checking Global Domain Assignments

  if [ ! -z $active_cmas ] && [ ! -z $MDSDIR ]; then  #running puv only on active user domains in MDS
    global_assignment=$(psql_client cpm postgres -t -c "select name from domainbase_data where dlesession=0 and not deleted and objid in (select objectstoredomain_objid from objectstoredomain_peerdomains_data where dlesession=0 and not deleted and peerdomain='1e294ce0-367a-11e3-aa6e-0800200c9a66' and version<>(select currentversion from domainversion where domainid='1e294ce0-367a-11e3-aa6e-0800200c9a66') and objectstoredomain_objid in ($active_cmas));" 2>/dev/null)
     if [ $? -ne 0 ]; then
	     $ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
	     exit 4
     fi
     if [ ! -z "$global_assignment" ]; then
	     output="${output}\n$i. The following domains are not assigned to latest Global domain version. Perform Reassign Global Policy and run the operation again. Domains: $global_assignment."
	     i=$((i+1))
     fi
  fi
  
  # 3. Check for valid license
  check_valid_license
  if [ $? -ne 0 ]; then
		  output="${output}\n$i. The server does not have a valid license. For more information on how to update your license. For assistance with your license, contact Check Point Account Services: https://help.checkpoint.com."
	  i=$((i+1))
  fi
  
  # 4. 
  currentVersion=$(find_current_version)
  upgradeToolsVersion=$(find_upgrade_tools_version)
  if [ "X$currentVersion" = "XR80.20" -a "X$upgradeToolsVersion" = "XR80.30" -a ! -f $ALLOW_R80_20_UPGRADE_FLAG_FILE ]; then 
	output="${output}\n$i. The migrate_server command is not supported for an upgrade from R80.20 to R80.30. For more information, see the \"Check Point R80.30 Installation and Upgrade Guide\"."
	i=$((i+1))
  fi

  # 5. MultiSite - Global active on secondary
    isMds=1
    if [ "X$MDSDIR" = "X" ]; then
        isMds=0
    fi

    active=`$CPPROD_UTIL FwIsActiveManagement`;
    if [ $? -ne 0 ]; then
        $ECHO "Could not run compatibility check. Failed to read 'FwIsActiveManagement' value from the registry. Please contact Check Point Support for further assistance."
	      exit 17
    fi
    primary=`$CPPROD_UTIL FwIsPrimary`;
    if [ $? -ne 0 ]; then
        $ECHO "Could not run compatibility check. Failed to read 'FwIsPrimary' value from the registry. Please contact Check Point Support for further assistance."
	      exit 18
    fi
    $ECHO "$(date) $c active $active primary $primary" >> $LOG_FILE 2>&1
    if [ $active -ne $primary ]; then
      if [ $isMds == 1 ]; then
        output="${output}\n$i. To allow the upgrade operation, make sure that Global Domain Management is Active on the primary machine, and Standby on all secondary machines."
        i=$((i+1))
      else
        output="${output}\n$i. To allow the upgrade operation, make sure that the primary server is Active, and the secondary servers are on Standby."
        i=$((i+1))
      fi
    fi

  # 6. Check for IPS protections links
  
	# links to exception rules

     if [ ! -z $active_cmas ]; then  #running puv only on active user domains
	   protections_list=$(psql_client cpm postgres -t -c "select STRING_AGG(distinct ('protection: \"' || cpmidisplayname || '\" in domain: \"' || dd.name || '\" '), ', ') from dbindexsdtopicperprofiledynamic_data as db inner join dleobjectderef_data as dle on db.objid=dle.objid inner join link_data as ld on ld.destinationobjectid=dle.objid inner join domainbase_data as dd on dd.objid=ld.domainid where ld.domainid in ($active_cmas) and dle.name in ('SDT_msrpc_over_cifs_protection','SDT_snmp_always_on','SDT_asm_dynamic_prop_srat','SDT_asm_content_protos','SDT_asm_web_clients_protections','SDT_asm_dynamic_prop_always_on','SDT_asm_remote_control_protos','SDT_ftp_patterns_word_list','SDT_asm_routing_protos','SDT_asm_dynamic_prop_http_hand','SDT_asm_sun_rpc_protos','SDT_asm_dynamic_prop_icq_http','SDT_asm_NGX_BC_protos','SDT_DCERPC_GLOBAL_DROP_NO_AUTH','SDT_asm_veritas_protos','SDT_malformed_imap_commands') and db.dlesession=0 and not db.deleted and dle.dlesession=0 and not dle.deleted and not db.hidden and ld.linkschemaid='2c4d7850-4ee9-3284-9651-73f56e1f3bdb' and ld.dlesession=0 and not ld.deleted;" 2>/dev/null)
	   if [ ! -z "$protections_list" -a "$protections_list" != " " ]; then
		output="${output}\n$i. The following IPS protections cannot be referenced from an exception rule. Remove the protections from the relevant exception rules and run the operation again. IPS Protections: $protections_list."
		i=$((i+1))
	   fi
     fi

#prevent an upgrade from R80.20/R80.30 to R80.40 in case when Postgres DB contain UTM-1 / EDGE objects.
	#if MDS - join two tables - one for object name, the second for cma name.
	if [ "X$MDSDIR" != "X" ]; then
		objects=$(psql_client cpm postgres -t -c "select distinct FORMAT('%s (Domain: %s), ',dletable.name, domainbase_data.name) from (select name, domainid from dleobjectderef_data where dlesession=0 and deleted=false and (objclass ='com.checkpoint.objects.classes.dummy.CpmiSofawareGateway' or objclass='com.checkpoint.objects.classes.dummy.CpmiLsmGatewayProfile')) as dletable INNER JOIN domainbase_data ON(dletable.domainid = domainbase_data.objid and domainbase_data.name!='System Data' and domainbase_data.objid!='a0bbbc99-adef-4ef8-bb6d-defdefdefdef');")
	else
		objects=$(psql_client cpm postgres -t -c "select distinct FORMAT('%s, ',dletable.name) from (select name, domainid from dleobjectderef_data where dlesession=0 and deleted=false and (objclass ='com.checkpoint.objects.classes.dummy.CpmiSofawareGateway' or objclass='com.checkpoint.objects.classes.dummy.CpmiLsmGatewayProfile')) as dletable INNER JOIN domainbase_data ON(dletable.domainid = domainbase_data.objid and domainbase_data.name!='System Data' and domainbase_data.objid!='a0bbbc99-adef-4ef8-bb6d-defdefdefdef');")
	fi		
	if [ ! -z "$objects" ]; then
		#count the numbers of words(name of object cannot contain spaces)
		sizeofobjects=`$ECHO "${objects}" | wc -w`
		objects="${objects::-2}."
		if [ "X$MDSDIR" != "X" ]; then
			#count the numbers of "(" - (name of cma can contain spaces)
			char="("
			sizeofobjects=`$GREP -o "(" <<<"$objects" | wc -l`
		fi
		output="${output}\n$i.UTM-1 Edge Gateways, LSM UTM-1 Edge Profiles (in SmartConsole) and UTM-1 Edge Gateways (in SmartProvisioning) must be deleted prior to proceeding with the Management upgrade process. Note: Existing UTM-1 Edge Gateways (from SmartProvisioning) are not printed below."
		i=$((i+1))
		
		#if there are less than 10 objects, print the names
		if [ "$sizeofobjects" -le 10 ]
			then 
			output="$output\nThese are the UTM-1 Edge Gateways, LSM UTM-1 Edge Profiles that currently exist on the management: $objects"
				
		fi
	fi


	 # 7 + 8 . ONLY IN CPUSE
  initilize_mds_primary_data()
  {
    primaryName=$(psql_client cpm postgres -t -c " select name from dleobjectderef_data where objclass='com.checkpoint.objects.mdsclasses.dummy.CpmiMds' and fwset ilike '%:primary (true)%' and dlesession=0 and not deleted;")
    if [ $? -ne 0 ]; then
      $ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
      exit 7
    fi
    primaryName=`$ECHO "$primaryName" | /bin/sed 's/ //g' `
    primaryLegacyObject=$(psql_client cpm postgres -t -c " select objid from dleobjectderef_data where objclass='com.checkpoint.objects.mdsclasses.dummy.CpmiMds' and fwset ilike '%:primary (true)%' and dlesession=0 and not deleted;")
    if [ $? -ne 0 ]; then
      $ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
      exit 7
    fi
    primaryLegacyObject=`$ECHO "$primaryLegacyObject" | /bin/sed 's/ //g' `
    primaryIp=$(psql_client cpm postgres -t -c " select ipaddress4  from  cpnetworkobject_data where dlesession=0 and not deleted and legacyobject='$primaryLegacyObject' and dlesession=0 and not deleted;")
    if [ $? -ne 0 ]; then
      $ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
      exit 7
    fi
    primaryIp=`$ECHO "$primaryIp" | /bin/sed 's/ //g' `
  }

  initilize_smc_primary_data(){
    primaryName=$(psql_client cpm postgres -t -c "select name from dleobjectderef_data where (objclass='com.checkpoint.objects.classes.dummy.CpmiHostCkp' or objclass='com.checkpoint.objects.classes.dummy.CpmiGatewayCkp') and fwset ilike '%:primary_management (true)%' and dlesession=0 and not deleted;")
    if [ $? -ne 0 ]; then
      $ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
      exit 7
    fi
    primaryName=`$ECHO "$primaryName" | /bin/sed 's/ //g' `
    primaryIp=$(psql_client cpm postgres -t -c "select ipaddress4 from  cpnetworkobject_data where name='$primaryName' and dlesession=0 and not deleted")
    if [ $? -ne 0 ]; then
      $ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
      exit 7
    fi
    primaryIp=`$ECHO "$primaryIp" | /bin/sed 's/ //g' `
  }



  if [ "X$UPGRADE_CONTEXT" = "Xcpuse" ]; then
    $ECHO "$(date) starting primary PUV" >> $LOG_FILE 2>&1
    isPrimaryChecked=0
    primaryChangedIp=0

    # 7. In MDS secondary/ MLM check that the primary is upgraded successfully
		isPrimary=`$CPDIR/bin/cpprod_util FwIsPrimary| /bin/sed 's/ //g'`;
          if [ $? -ne 0 ]; then
              $ECHO "Could not run compatibility check. Failed to read 'FwIsPrimary' value from the registry. Please contact Check Point Support for further assistance."
	            exit 6
          fi
      if [ "X1" != "X$isPrimary" ]; then
          if [ ! -e "$SKIP_PRIMARY_CHECK_FLAG_FILE" ]; then
              skipCheckFlag=0
              if [ "X$MDSDIR" != "X" ]; then
                initilize_mds_primary_data
              else
                initilize_smc_primary_data
              fi
              $ECHO "$(date) Primary machine $primaryName :  $primaryIp " >> $LOG_FILE 2>&1
              if [ "X" == "X$primaryIp" -o "X" == "X$primaryName" ]; then
                skipCheckFlag=1
              fi
              if [ -e "$CHANGED_IP_FILE" -a $skipCheckFlag -eq 0 ]; then
                changeIpFile=`jq '.[] | .name, .newIpAddress4' $CHANGED_IP_FILE`
                changeIpFileArray=( $changeIpFile )
                isName=1
                name=""
                for data in "${changeIpFileArray[@]}"
                do
                	# i is the name- the odd positioned elements are the names
                	if [ $isName -eq 1 ]; then
                	  name=$data
                	  isName=0
                	# i is the newIpAddress4
                	else
                	  isName=1
                	  ip=$data
                	  $ECHO "Machine $name changed ip to $ip " >> $LOG_FILE 2>&1
                	  #If the ip of the primary in the file is he same as in the DB- the file is not relevant for the current upgrade
                          #If the ip is different- the primary is upgraded and changed it's ip- skip check.
                	  if [ \""$primaryIp"\" != "$ip" -a \""$primaryName"\" == "$name" ]; then
                      primaryIp=$ip
                      primaryIp=${primaryIp%?}   #Remove last "
                      primaryIp="${primaryIp:1}"   #Remove first "
                      primaryChangedIp=1
                      break;
                    fi
                	fi
                done
              fi
              if [ $skipCheckFlag -eq 0 ]; then
                primaryOs=`cprid_util -server $primaryIp rexec -verbose  -rcmd cpstat os 2>> $LOG_FILE`
                cpridStatus=$?
                if [ $cpridStatus -eq 5 ]; then
                  if [ $primaryChangedIp -eq 0 ]; then
                   output="${output}\n$i. Upgrade operation is not supported in the current configuration.  Make sure that SIC is established with the primary machine and try again. Contact Check Point Support for further assistance.”"
                   i=$((i+1))
                  else #If there is no sic, and the primary changed ip- it probably means that the primary was not upgraded yet.
                    output="${output}\n$i. Upgrade the primary machine first. Make sure the upgrade is successful and the primary machine is up and running, and that SIC is established with it, before you try again."
                    i=$((i+1))
                  fi
                fi
                if [ $cpridStatus -eq 0 ]; then
                  $ECHO "The os of the primary machine: $primaryOs" >> $LOG_FILE 2>&1
                  containsVersion=`$ECHO "$primaryOs" | grep "SVN Foundation Version String:"`
                    if [ "X" != "X$containsVersion" ]; then
                      primaryVersion=`$ECHO "$primaryOs" | grep "SVN Foundation Version String: $upgradeToolsVersion"`
                      if [ "X" = "X$primaryVersion" ]; then
                        output="${output}\n$i. Upgrade the primary machine first. Make sure the upgrade is successful and the primary machine is up and running before you try again."
                        i=$((i+1))
                      else
                        isPrimaryChecked=1
                      fi
                  fi
                fi
              fi
          fi
      fi

      # 8. In MLM check that all of the relevant active domains are upgraded successfully
      if [ $isPrimaryChecked -eq 1 -a "X$MDSDIR" != "X" ]; then
          isMDS=`$CPDIR/bin/cpprod_util CPPROD_GetValue PROVIDER-1 LoginServer 1`

          if [ $? -ne 0 ]; then
              $ECHO "Could not run compatibility check. Failed to read 'LoginServer' value from the registry. Please contact Check Point Support for further assistance."
              exit 12
          fi
          if [ "X0" = "X$isMDS" ]; then #This is a MLM
            currentMdsSicName=`$CPDIR/bin/cpprod_util CPPROD_GetValue SIC MySICname 1`
	        if [ $? -ne 0 ]; then
                  $ECHO "Could not run compatibility check. Failed to read 'MySICname' value from the registry. Please contact Check Point Support for further assistance."
                  exit 13
          fi
	        currentMdsId=$(psql_client cpm postgres -t -c "select objid from cpnetworkobject_data where dlesession=0 and not deleted and sicname='$currentMdsSicName';"| sed -e 's/^[[:space:]]*//' |sed -r '/^\s*$/d')
          if [ $? -ne 0 ]; then
                  $ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
	              exit 14
          fi
	        localDomains=$(psql_client cpm postgres -t -c "select objid from domainbase_data where dtype = 'LocalDomain' and objid in (select domain from domainserver_data where dlesession=0 and not deleted and mds='$currentMdsId') and dlesession=0 and not deleted;" | sed -e 's/^[[:space:]]*//' |sed -r '/^\s*$/d')
          if [ $? -ne 0 ]; then
                  $ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
	              exit 15
          fi
	        LocalDomainsArray=(${localDomains//'\n'/ })
	        notUpgraded=""
			    localDomainsList="" # Will contain a list of local domains, that can be used in a query.
          for dom in "${LocalDomainsArray[@]}"
	        do
				    localDomainsList+="'"
				    localDomainsList+="$dom"
				    localDomainsList+="',"
	        done
  			    if [ ! -z $localDomainsList ]; then
	  			    localDomainsList=${localDomainsList%?} #Remove last comma
		  		    upgradedDomains=`cprid_util -server $primaryIp rexec -verbose  -rcmd psql_client cpm postgres -t -c "select objid from domainbase_data where objid in ($localDomainsList) and  dlesession=0 and not deleted;" | sed -e 's/^[[:space:]]*//' |sed -r '/^\s*$/d' 2>> $LOG_FILE`
		          cpridStatus=$?
				      if [ $cpridStatus -eq 5 ]; then
					      output="${output}\n$i. Upgrade operation is not supported in the current configuration.  Make sure that SIC is established with the primary machine and try again. Contact Check Point Support for further assistance.”"
  					      i=$((i+1))
	  			    fi
		  		    if [ $cpridStatus -eq 0 ]; then
			  		    for dom1 in "${LocalDomainsArray[@]}"
				  	    do
					  	    if [[ ! $upgradedDomains =~ $dom1 ]]; then
						  	    notUpgraded+="'"
							      notUpgraded+="$dom1"
							      notUpgraded+="',"
  						    fi
	  				    done

	  			    fi
		  		    if [ "X" != "X$notUpgraded" ]; then
			  		    notUpgraded=${notUpgraded%?} #Remove last comma
				  	    domName=$(psql_client cpm postgres -t -c "select name from domainbase_data where objid in ($notUpgraded) and dlesession=0 and not deleted;" | sed -e 's/^[[:space:]]*//' |sed -r '/^\s*$/d')
				        if [ $? -ne 0 ]; then
				          $ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
				          exit 16
			          fi
						  output="${output}\n$i. Before you can upgrade this Multi-Domain Log Server, you must upgrade all Multi-Domain Servers whose managed Security Gateways send logs to this Multi-Domain Log Server.\n     If all Multi-Domain Servers are already upgraded, wait for a few minutes to allow the Primary and Secondary Multi-Domain Servers to synchronize. After that time, start the upgrade again.\n     If the issue persists, contact Check Point Support.\nThese Domains are not yet synchronized with the Primary Server: $domName"
					      i=$((i+1))
				      fi
			      fi
          fi



      fi
  fi


	# 9. Block upgrade in SMC-Backup if active.
	if [ "X$MDSDIR" = "X" ]; then # SMC mode
		isSMCBackup=`$CPPROD_UTIL FwIsSMCBackup`; # Check if it's SMC-Backup from registry file.
		if [ $? -ne 0 ]; then
                    $ECHO "Could not run compatibility check. Failed to read 'FwIsSMCBackup' value from the registry. Please contact Check Point Support for further assistance."
                    exit 1
                fi
		if [ "$isSMCBackup" = "1" ]; then
			active=`$CPPROD_UTIL FwIsActiveManagement`;
			if [ $? -ne 0 ]; then
                            $ECHO "Could not run compatibility check. Failed to read 'FwIsActiveManagement' value from the registry. Please contact Check Point Support for further assistance."
                            exit 1
                        fi
			if [ $active -ne 0 ]; then
                            output="${output}\n$i. Upgrade operation is not supported in current configuration.  Make sure that SMC-Backup is not in active state and try again."
                            i=$((i+1))
			fi
		fi
	fi
				
	# 10. Block upgrade if root partition has less than 1G free
	free_space=`df -h / | awk '/[0-9]%/{print $(NF-2)}'` >> $LOG_FILE 2>&1
	if [ $? -ne 0 ]; then
      $ECHO "Could not run compatibility check. Failed to run ‘df-h’ command. Please contact Check Point Support for further assistance."
	    exit 8
  fi
	$ECHO "$(date) The root partition has $free_space free space" >> $LOG_FILE 2>&1
	if [ "X0G" = "X$free_space" -o "X0" = "X$free_space" ]; then
	  				output="${output}\n$i. Upgrade blocked because your root partition is full. Please extend it and try again."
				i=$((i+1))
	fi



	# 10. Run on active domains. Check if there exists ELIS with 'SECTION' as entity that does not have a matching object in the relevant section table.
		if [ ! -z $active_cmas ]; then
			corruption_detected=$(psql_client cpm postgres -t -c "	SELECT entity FROM rulebaseentitylocalinstance_data WHERE dlesession=0 AND not deleted
								AND entitytype='SECTION' AND owner IN (SELECT objid FROM dleobjectderef_data WHERE dleobjectderef_data.objclass='com.checkpoint.objects.threat_rulebase.ThreatExceptionRulebase')
					AND entity NOT IN (SELECT objid FROM threatrulebasesection_data);" 2>/dev/null)
			if [ ! -z "$corruption_detected" -a "$corruption_detected" != " " ]; then
				output="${output}\n$i. Threat Section Corruption Detected, please see sk170553 for more information"
				i=$((i+1))
			fi
		fi


	
	
	
	# 11. In case upgradeWithoutIps file exists, check that there are no links from threat exceptions, and threat profiles to IPS domain.
	#Running puv only on active user domains and global domain in MDS

	FILE=/var/log/upgradeWithoutIps
	if [ -f "$FILE" ]; then
		$ECHO "exsists" 
		if [ ! -z $active_cmas ]; then
			exceptions_list=$(psql_client cpm postgres -t -c "select STRING_AGG(distinct ('protection: \"' || cpmidisplayname || '\" in domain: \"' || dd.name || '\" '), ', ') from dbindexsdtopicperprofiledynamic_data as db inner join dleobjectderef_data as dle on db.objid=dle.objid inner join link_data as ld on ld.destinationobjectid=dle.objid inner join domainbase_data as dd on dd.objid=ld.domainid where db.dlesession=0 and not db.deleted and dle.dlesession=0 and not dle.deleted and not db.hidden and ld.linkschemaid='2c4d7850-4ee9-3284-9651-73f56e1f3bdb' and ld.dlesession=0 and not ld.deleted and (dd.objid in ($active_cmas) or dd.objid = '1e294ce0-367a-11e3-aa6e-0800200c9a66');" 2>/dev/null)
			if [ ! -z "$exceptions_list" -a "$exceptions_list" != " " ]; then
				output="${output}\n$i. In upgrade with /var/log/upgradeWithoutIps, no IPS protections cannot be referenced from an exception rule. Remove the protections from the relevant exception rules and run the operation again. IPS Protections: $exceptions_list."
				i=$((i+1))
			fi
		
			profiles_list=$(psql_client cpm postgres -t -c "select STRING_AGG(distinct ('profile: \"' || d.name || '\" in domain: \"' || dd.name || '\" '), ', ') from link_data l join domainbase_data as dd on dd.objid=l.domainid join dleobjectderef_data d on l.sourceobjectid = d.objid join dleobjectderef_data dle on l.destinationobjectid = dle.objid where d.objclass ilike '%CpmiArmadaNewProfile' and dle.objclass ilike '%CpmiIpsTags' and not l.deleted and l.domainid<>'a0bbbc99-adef-4ef8-bb6d-defdefdefdef' and (dd.objid in ($active_cmas) or dd.objid = '1e294ce0-367a-11e3-aa6e-0800200c9a66');" 2>/dev/null)
			if [ ! -z "$profiles_list" -a "$profiles_list" != " " ]; then
				output="${output}\n$i. In upgrade with /var/log/upgradeWithoutIps, no IPS tags can be referenced from a user created threat profile (in IPS-> Additional Activation). Remove the tags from the relevant profiles rules and run the operation again. Profiles: $profiles_list."
				i=$((i+1))
			fi
		fi
	fi


	#12. Block upgrade when a domain has more than one active domain server
	if [ "X$MDSDIR" != "X" ]; then #Run only in MDS for now
   isMDS=`$CPDIR/bin/cpprod_util CPPROD_GetValue PROVIDER-1 LoginServer 1`
   if [ $? -ne 0 ]; then
        $ECHO "Could not run compatibility check. Failed to read 'LoginServer' value from the registry. Please contact Check Point Support for further assistance."
        exit 12
   fi
   if [ "X0" != "X$isMDS" ]; then #This is not a MLM
	  server_restart_versions=("R80.20" "R80.20.M1" "R80.20.M2" "R80.30" "R80.40")
  	#For old versions- check aggregators
    if [[ "${server_restart_versions[@]}" =~ "${currentVersion}" ]]; then


      #Aggregators of local domains with more then one active domain server- delimited  by $
      aggs=$(psql_client cpm postgres -t -c " select  FORMAT('%s$ ',object) from dleobjectwrapper_data where object ilike '%domainaggregator%' and object ilike '%\""domainType\"":\""LOCALDOMAIN\""%' and object ilike '%\""mode\"":\""ACTIVE\""%"mode\"":\""ACTIVE\""%' and dlesession = 0 and not deleted;")
      if [ $? -ne 0 ]; then
        $ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
        exit 16
      fi
      agg=${aggs//[$'\t\r\n']} #Remove tabs and new lines so we will iterate over all of the aggregators

      IFS='$'  #Use the spacial character '$' as delimiter - since it is not allowed in domain and domainserver name.
      read -a aggArray <<< "$agg"  #Create a list of aggregators
      unset IFS
      active_active=""

      for dom in "${aggArray[@]}"
      do
  	    content=`$ECHO "$dom"  | jq '.[1]' `
  	    domain=`$ECHO "$content" | jq '.domain' `
    	  name=`$ECHO "$domain" | jq '.[1]' | jq '.name' `
    	  if [ "X" != "X$name" ]; then
      		active_active+=" $name,"
    	  fi
     done

     check_global=$(psql_client cpm postgres -t -c " select  FORMAT('%s$ ',object) from dleobjectwrapper_data where object ilike '%domainaggregator%' and object ilike '%\""domainType\"":\""GLOBALDOMAIN\""%' and object ilike '%\""mode\"":\""ACTIVE\""%"mode\"":\""ACTIVE\""%' and dlesession = 0 and not deleted;")
      if [ $? -ne 0 ]; then
        $ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
        exit 16
      fi

      if [ "X" != "X$check_global" ]; then
      	active_active+=" \"global\","
      fi
    fi
 fi

  #For R81 and above- check domainserver.active
  else


    active_active=$(psql_client cpm postgres -t -c "select FORMAT(' %s,' , name) from domainbase_data DB where dtype in ('GlobalDomain','LocalDomain') and 2<= (select count(*) from domainserver_data where active='1' and domain=DB.objid and dlesession=0 and not deleted group by domain) and dlesession=0 and not deleted;")
    if [ $? -ne 0 ]; then
      $ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
      exit 16
    fi

    active_active=${active_active//[$'\t\r\n']} #Remove tabs and new lines

  fi

  if [ "X" != "X$active_active" ]; then
  	active_active=${active_active%?} #Remove last comma
	  output="${output}\n$i. Upgrade operation is not supported when a domain has more than one active domain server. \n     Select one active domain server for each domain and set the rest to standby. \n     Relevant Domains : $active_active"
    i=$((i+1))
  fi

  # 13. Check DLS trust files exists
  if [ "X$MDSDIR" = "X" ]; then
	#Check DLS trust files exists on SMC mode (SMCB or DLS where crldp is UID format)
        trustedCasExists=$(psql_client cpm postgres -t -c "select exists (select 1 from information_schema.tables where table_name='trustedca_data');" | sed -e 's/^[[:space:]]*//')
        if [ $? -ne 0 ]; then
             $ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
             exit 9
        fi
	isSMCBackup=`$CPPROD_UTIL FwIsSMCBackup`; # Check if it's SMC-Backup from registry file.
	if [ $? -ne 0 ]; then
            exitAndLog 1 "Could not run compatibility check. Failed to read 'FwIsSMCBackup' value from the registry. Please contact Check Point Support for further assistance."
	fi
		
	if [ "$isSMCBackup" == "1" -o "$isLogServer" == "1" ]; then
		if [[ "$trustedCasExists" == "t"  && -f "$CPDIR/conf/sic_cert.p12" ]]; then
			crlDpURI=`cpopenssl pkcs12 -in $CPDIR/conf/sic_cert.p12 -nokeys -nomacver -passin pass: | cpopenssl x509 -noout -text | grep .crl`
			hasUUIDCrlDP=`echo $crlDpURI | grep .crl | grep 'URI:http' | egrep  "URI:http://[{]?[0-9a-fA-F]{8}-([0-9a-fA-F]{4}-){3}[0-9a-fA-F]{12}[}]?:18264"`
			#For log server using multi SIC crl format
			if [ $? -eq 0 ]; then
				multiSicTrustFiles=(
				"SIC_DB"
				"crl_distribution_point.map"
				"sic_ca_bundle.pem"
				)
				for sicFile in "${multiSicTrustFiles[@]}"
				do
					if [ ! -f "$CPDIR/conf/$sicFile" ]; then
						$ECHO "Operation failed. File: $CPDIR/conf/$sicFile does not exist, file is required to support Log Server connectivity. Please see sk172329 for more information."
						output="${output} $i. Operation failed. File: $CPDIR/conf/$sicFile does not exist, file is required to support Log Server connectivity. Please see sk172329 for more information."
						i=$((i+1))
					fi
				done
			fi
		fi
	fi
  fi


	#14. Block upgrade for MDS server - if SMC Backup exists, domain must have an active domain server
	
	if [ ! -z $MDSDIR ] ; then
		isMDS=`$CPDIR/bin/cpprod_util CPPROD_GetValue PROVIDER-1 LoginServer 1`
		if [ $? -ne 0 ]; then
			exitAndLog 20 "Could not run compatibility check. Failed to read 'LoginServer' value from the registry. Please contact Check Point Support for further assistance."
		fi
	
		smcbHosts=`psql_client cpm postgres -t -c "select distinct name from domainbase_data where objid in (select a.domain from ((select domain,count(*) as count from domainserver_data where active='false' and not deleted and dlesession=0 group by domain) as a inner join (select domain,count(*) as count from domainserver_data where not deleted and dlesession=0 group by domain) as b on a.domain=b.domain) where a.count=b.count and a.domain in (select distinct domain from domainserver_data where servertype=2) and a.domain not in ('a0eebc99-afed-4ef8-bb6d-fedfedfedfed'))" | sed -e 's/^[[:space:]]*//' |sed -r '/^\s*$/d'` 
		if [ $? -ne 0 ]; then
		  exitAndLog 20
		fi
		if [ "X$smcbHosts" != "X" ]; then
			output="${output}\n$i. Some domains are configured in Management High Availability with a SmartCenter that is set as the active peer.\n  To continue:\n  (1) Connect with SmartConsole to each such Domain. \n     (2) Change the Domain Server to be the active peer.\n     These domains are configured in Management High Availability with a SmartCenter that is set as the active peer: $smcbHosts"
			i=$((i+1))
		fi
	fi
	
	# 15. Block upgrade in case global isn't active and there are duplicate objIds in GVC objects in active domains
	if [ ! -z $MDSDIR ] && [ ! -z $active_cmas ]; then 
		isPrimary=`$CPDIR/bin/cpprod_util FwIsPrimary| /bin/sed 's/ //g'`;
        if [ $? -ne 0 ]; then
              $ECHO "Could not run compatibility check. Failed to read 'FwIsPrimary' value from the registry. Please contact Check Point Support for further assistance."
	          exit 21
        fi
		if [ "X1" != "X$isPrimary" ]; then #means the global isn't active
			gvc_classes="'com.checkpoint.management.classes.dle.gvc.EncryptionDomainHolderMirrorObject', 'com.checkpoint.management.classes.dle.gvc.InternalCaServerMirrorObject', 'com.checkpoint.management.classes.dle.gvc.ObjectCkpMirrorObject', 'com.checkpoint.management.ngm_auth_keys.objects.BlobAuthKeysMirrorObject'"
			duplicateObjIds=$(psql_client cpm postgres -t -c "select m.objid  from mirrorobjectindex m , dleobjectderef_data dod where dod.objid = m.mirrorobjectid and dod.dlesession=0 and not dod.deleted and m.originaldomainid in ($active_cmas) and dod.objclass in ($gvc_classes) and m.objid in (select objid from dleobjectderef_data where dlesession = 0 and not deleted group by objid having count(*) > 1);" 2>/dev/null)
			if [ $? -ne 0 ]; then
				$ECHO "Could not run compatibility check. Please contact Check Point support for further assistance."	
				exit 21
			fi
			if [ ! -z $duplicateObjIds ]; then
				output="${output}\n$i. There are Global VPN Community objects that sharing the same object ID. Please contact Check Point Support for further assistance."
				i=$((i+1))
			fi
		
		fi
	fi
	
	# 16. Verify that thre are IPS packages suitable for the export
	#Running puv only on active user domains and global domain in MDS

	if [ ! -z $active_cmas ]; then
		ips_worksessions_list=$(psql_client cpm postgres -t -c "select w.objid from worksession w inner join domainworksession d on w.objid=d.worksessionid where d.dledomainid='a0bbbc99-adef-4ef8-bb6d-cebcebcebceb'and w.state ='PUBLISHED' and d.sessionversion <> -1 and (w.objid in (select worksessionuid from ipsupdatepackagemap_data where ipspackageversion=635158746) or (w.objid in (select w.objid from worksession w inner join domainworksession d on w.objid=d.worksessionid where d.dledomainid='a0bbbc99-adef-4ef8-bb6d-cebcebcebceb'and w.state ='PUBLISHED' and d.sessionversion <> -1 and d.dlesession=0 order by d.publishtime DESC limit 1)) or (w.objid in (select a.globalworksessionid from globaldomainassignment_data a where a.globaldomain='a0bbbc99-adef-4ef8-bb6d-cebcebcebceb' and dlesession=0 and deleted=false))) and w.objid in (select worksessionuid from ipsupdatepackagemap_data where deleted=false ) order by d.publishnumber;" 2>/dev/null)
		if [ -z "$ips_worksessions_list" ]; then
			output="${output}\n$i. There are no IPS packages suitable for the upgrade. Do an IPS update before exporting the database. If you do not use IPS, follow sk168614."
			i=$((i+1))
		fi
	fi

	# 17. Block upgrade in case sic_cert.12 file is not exists
        if [ "X$MDSDIR" = "X" ]; then
          if [ "X$isManagement" == "X1" -o "X$isLogServer" == "X1" ]; then
            if [ ! -f "$CPDIR/conf/sic_cert.p12" ]; then
              if [ "X$isPrimary" != "X1" ]; then
                # file is not exists
                output="${output}\n$i. The file $CPDIR/conf/sic_cert.p12 is missing."
                i=$((i+1))
              fi
            fi
          fi
         fi

  # 19. Duplicated CpNetworkObject With The Same Sicname
  if [ "X$isManagement" == "X1" ]; then
    object_sicname=$(psql_client cpm postgres -t -c "select objid, name, sicname from cpnetworkobject_data where sicname in ( select sicname from cpnetworkobject_data where dlesession=0 and not deleted and sicname is not null and sicname != '' group by sicname having count(*)>1 ) and dlesession=0 and not deleted;" 2>/dev/null)
    if [ $? -ne 0 ]; then
      $ECHO "Could not run compatibility check. Please verify all server processes are up. If you require assistance, contact Check Point Support."
      exit 22
    fi
    if [ ! -z "$object_sicname" -a "$object_sicname" != " " ]; then
      output="${output}\n$i. There is more than one CpNetworkObject with the same sicname."
      i=$((i+1))
    fi
  fi

	
  # Stop postgres
  if [ $POSTGRES_WAS_DOWN -eq 1 ];then
        $ECHO "==================================================$MDS_FWDIR/scripts/postgres_stop.sh" >> $LOG_FILE 2>&1
        $MDS_FWDIR/scripts/postgres_stop.sh  >> $LOG_FILE 2>&1
  fi
fi


if [ "$output" == "" ]
then
	  $ECHO $SUCCESS_MSG
	  exit 0
else
	  $ECHO $output
	  exit 1
fi


exit 0