#!/bin/sh

. /opt/CPshared/5.0/tmp/.CPprofile.sh >> /dev/null

parse_args()
{
	#vars:
	debug=0
	log_location="$MDS_FWDIR/log/verification_report.elg"
	flags=""
	mode=""
	delimiter=",\n"
	
	#parse:
	while [ $# -gt 0 ]
	do
        case $1 in
		    -debug)
                debug=1
                ;;
			-log_location)
				debug=1
				shift
				if [ $# -eq 0 ]; then
					usage
					exit 1
                else
					log_location="$1"
				fi
				;;
			-delimiter)
				shift
				if [ $# -eq 0 ]; then
					usage
					exit 1
                else
					delimiter="$1"
				fi
				;;
            -mode)
				shift
				if [ $# -eq 0 ]; then
                   usage
				   exit 1
                fi
                mode="$1"
                ;;
			--help)
				help_print
				exit 0
				;;
            *)
                usage
				exit 1

        esac
        shift
	done
	if [ $debug -eq 1 ]; then
		debug_echo "mode is $mode"
	fi
}

function usage () {
	echo "Use the following parameters: [-debug] [-log_location <logger location>]  [-delimiter <symbol>] [-mode <export/upgrade>] [--help]"
}
function help_print(){
	echo "This script applies the update_inspect_files tool over all the CMAs."
	echo "In each CMA, checks if there are modified inspect files in the management lib and all its CMPs"
	echo "This script is meant to be run during the PUV checks, and its usage is as follows:"
	usage
}
function debug_echo() {
	if [ $debug -eq 1 ]; then
		if [ $# -gt 1 ]; then
			echo "DEBUG: $1" >> $2 2>&1
		else
			echo "DEBUG: $1" >> $log_location 2>&1
		fi
	fi
}


function get_cmp_products(){
	products="FW1"
	get_value="$CPDIR/bin/cpprod_util CPPROD_GetValue"

	for prod in `$CPDIR/bin/cpprod_util CPPROD_GetInstalledProducts`
	do
		eval "$get_value $prod FW1_BC_VER 1" >>/dev/null 2>/dev/null
		
		if [ $? -ne 0 ];then
			debug_echo "The product $prod is NOT CMP"
			continue
		fi
		debug_echo "The product $prod is CMP"
		products="$products $prod"
	done
}

 
function Packages_check() {
	result=""
	# products="FW1 R7520CMP  R7540CMP  R7540VSCMP  R76CMP  SFWR77CMP  R77CMP  R75CMP  NGXCMP  EdgeCmp  SFWCMP  FLICMP  SFWR75CMP"
	fw1_update_inspect_files=""
	
	for p in $products
	do
		PRODDIR=`$CPDIR/bin/cpprod_util CPPROD_GetProdDir $p| /bin/sed 's/ //g'`
		if [ ! -d "$PRODDIR" ]; then
			debug_echo "Failed to find $p package directory. Continue without changes"
			continue;
		fi

		if [ ! -d "$PRODDIR/lib" ]; then
			debug_echo "Product $p doesn't have 'lib' dir, skipping it"
			continue;
		fi

		UPDATE_INSPECT_FILES="$PRODDIR/bin/update_inspect_files"
    if [ ! -e $UPDATE_INSPECT_FILES  -o ! -x $UPDATE_INSPECT_FILES ]; then
      debug_echo "$UPDATE_INSPECT_FILES does not exist."
      UPDATE_INSPECT_FILES="$PRODDIR/bin/upgrade_tools/update_inspect_files"
      if [ ! -e $UPDATE_INSPECT_FILES  -o ! -x $UPDATE_INSPECT_FILES ]; then
        if [ "X$fw1_update_inspect_files" != "X" ]; then
          UPDATE_INSPECT_FILES="$fw1_update_inspect_files"
        else
          continue;
        fi
      fi
      debug_echo "Tool location has been changed to $UPDATE_INSPECT_FILES"
    fi
		
		if [ "X$p" == "XFW1" ]; then
			fw1_update_inspect_files="$UPDATE_INSPECT_FILES"
		fi

		flags="-path $PRODDIR/lib"
		#if [ "X$mode" != "X" ]; then
		#	flags+=" -mode $mode"
		#fi
	
		
		res=`$UPDATE_INSPECT_FILES $flags` >> $log_location 2>/dev/null
		stat=$?
#		if [ $stat -eq 0 ]; then
#      continue;
#    fi
		if [ $stat -ne 1 -a "X$mode" != "X" ] ; then
			flags+=" -mode $mode"
			res=`$UPDATE_INSPECT_FILES $flags` >> $log_location 2>/dev/null
			stat=$?
		fi
		debug_echo "$cma -> $p -> flags: ${flags}"
		if [ $stat -ne 1 ]; then
			debug_echo "Result: No inspect files were detected for $PRODDIR/lib"
			continue
		fi
		# In case there are inspect files
		debug_echo "Result: $res"
		# appends all the results together
		if [ "X$res" != "X" ] ; then ## an extra check for actually result
			if [ "X$result" == "X" ]; then
				result="$res"
			else
				result="$result$delimiter$res"
			fi
		fi
		#echo "$result"
	done
}

function domain_check(){
	active=`cpprod_util FwIsActiveManagement|sed "s/ //g"`;
	if [ $? -eq 0 -a "X$active" == "X1" ]; then
		Packages_check 2>/dev/null
		if [ "X$result" != "X" ] ; then ## an extra check for actually result
			if [ "X$over_all_answer" == "X" ]; then
				over_all_answer="$result"
			else
				over_all_answer="$over_all_answer$delimiter$result"
			fi
		fi
	fi
}

run(){
	debug_echo "------------------------------------"
	over_all_answer=""
	#$MDSVERUTIL AllCMAs | while read cma;
	for cma in `$MDSVERUTIL AllCMAs`
	do
		
		# Defines the current customar/domain as the working environment.
		mdsenv $cma
		
	#	active=`cpprod_util FwIsActiveManagement|sed "s/ //g"`;
		
		domain_check
		
	done

	# Returns to the global working environment.
	mdsenv
	
	#active=`$CPDIR/bin/cpprod_util CPPROD_IsProdActive PROVIDER-1`
	domain_check
	
}

parse_args $@

debug_echo "Starting update_inspect_files_wrapper"

get_cmp_products
debug_echo "The products list is: $products"


# Runs over all the domains
run     2>/dev/null
printf "$over_all_answer\n"
debug_echo "Done update_inspect_files_wrapper"
